from dataclasses import dataclass

import ts_Def as Def
import _stock_load as lib


class StockLoad:
#public:
    # 构造
    def __init__(self):
        self.c = lib.StockLoad()
        self.li = lib.Info()
        self.li2 = lib.Info2()
        self.lis2 = []

#public:
    # 设置加载起始路径
    def SetPath(self, path:str):
        self.c.SetPath(path)

    # 开始加载 yyyymmdd
    def BeginLoad(self, begin:int, end:int):
        self.c.BeginLoad(begin, end)

    # 获取一个信息
    def Get(self, si:Def.Info) -> bool:
        
        bGet = self.c.Get(self.li)
        if bGet:
            si.date = self.li.date
            si.time = self.li.time
            si.type = self.li.type
            si.mk   = self.li.mk
            si.code = self.li.code
            si.open = self.li.open
            si.high = self.li.high
            si.low  = self.li.low
            si.close= self.li.close
            si.volume = self.li.volume

        return bGet

    # 获取一个信息
    def Get2(self, si:Def.Info) -> bool:
        
        bGet = self.c.Get2(self.li2)
        if bGet:
            si.date = self.li2.date
            si.time = self.li2.time
            si.type = self.li2.type
            si.mk   = self.li2.mk
            si.code = self.li2.code
            si.open = self.li2.open
            si.high = self.li2.high
            si.low  = self.li2.low
            si.close= self.li2.close
            si.volume = self.li2.volume

        return bGet        

    # 获取一日信息
    def GetDay(self, sis) -> bool:
        sis.clear()
        bGet = self.c.GetDay(self.lis2)
        if bGet:
            sis.extend(self.lis2)
            self.lis2.clear()

        return bGet

    # 获取一日信息
    def GetDay2(self) -> bool:
        self.c.GetDay(self.lis2)
        return self.lis2

    # 是否获取完毕
    def IsOver(self) -> bool:
        return self.c.IsOver();    


# -*- coding: utf-8 -*-
from frame import data_center
from common import basefunc
import os
import pandas as pd


IDX_FILENAME = [basefunc.get_path_dirname(), "data", "ma", "{symbol}.xlsx"]

# isUp = 0 # 短线上穿长线信号 0表示短在长下
class MaMgr:
    def __init__(self,symbol= "SH.000300"):
        self.df = self.getIdxData(symbol)

    def getIsOpen(self,datetime): # 当日短线是否在长线上方
        isOpen = self.df.loc[datetime,"isOpen"]
        return isOpen

    def isDnCross(self,datetime): # 是否是死叉
        isOpen = self.getIsOpen(datetime)
        if isOpen: # 如果当日是短在上
            return False
        idx = self.df.index.tolist().index(datetime)
        if idx == 0:
            return False
        preDatetime = self.df.index.tolist()[idx-1]
        if self.getIsOpen(preDatetime): # 如果昨日短在上
            return True

    def getIdxData(self,symbol):
        fileName = os.sep.join(IDX_FILENAME)
        fileName = fileName.format(symbol = symbol)
        if not os.path.exists(fileName):
            self.datacenter = data_center.use()
            maShort = 100
            maLong = 200  # 均线指标
            beg_date = 20081231
            beg_date_idx = self.datacenter.trade_days.index(beg_date)
            beg_ma_day = self.datacenter.trade_days[beg_date_idx - maLong + 1]
            conn = self.datacenter.database.conn
            sql = "select time as date,code as '代码',close as '收盘价' from index_day_data where code ='%s' and time >=%s" % \
                  (symbol, beg_ma_day)
            df = pd.read_sql(sql, conn)

            for maDay in [maShort,maLong]:
                maColName = "ma" + str(maDay)
                df[maColName] = df["收盘价"].rolling(maDay).mean()

            def _calc(hang):  # 计算收盘价是否在当日50日均线下方
                # global isUp  # 是否上穿
                # close = hang["收盘价"]
                _maShort = hang["ma" + str(maShort)]
                _maLong = hang["ma" + str(maLong)]

                if _maShort>_maLong:
                    return 1
                return 0

            df["isOpen"] = df.apply(_calc, axis=1)
            df = df.set_index(["date"])
            df = df.dropna(axis=0)  # 删除带空值得行

            df.to_excel(fileName, sheet_name="指数数据")
            print("生成指数数据清单完成")

        else:
            df = pd.read_excel(fileName, sheet_name="指数数据", index_col=0)
            print("加载指数数据清单完成")

        return df

if __name__ == '__main__':
    obj = MaMgr("SH.000001")
    df = obj.getIsOpen(20190404)
    print(df)
    df = obj.getIsOpen(20200723)
    print(df)


# -*- coding: utf-8 -*-
import json
import os
import re
import sys
import requests
import datetime
import demjson
from functools import partial
from tradeTools import Decorator
import pandas as pd
import numpy as np


STOCK_CODE_PATH = 'stock_codes.conf'
# IDX_STOCKS={
# "HS300":"000300",
# "SZ50":"000016",
# "ZZ500":"000905"
# }

def code_add_prefix(code):
    """给没有前缀的股票代码增加前缀"""
    if isinstance(code, int):
        if code < 999999:
            code = '%06d' % (code)
        else:
            code = '%s' % (code)
               
    if code.startswith('SH.') or  code.startswith('SZ.'):
        return code
    return '%s.%s' % (get_stock_exchange(code), code)

def get_stock_exchange(stock_code):
    """判断股票ID对应的证券市场
    匹配规则
    ['50', '51', '60', '90','110'] 为 SH
    ['00', '13', '18', '15','16', '18', '20', '30', '39', '115'] 为 SZ
    ['5', '6', '9'] 开头的为 SH， 其余为 SZ
    :param stock_code:股票ID, 若以 'SZ', 'SH'开头直接返回对应类型，否则使用内置规则判断
    :return 'SH' or'SZ'"""
    if isinstance(stock_code, int) and stock_code < 999999:
        stock_code = '%06d' % (stock_code)
        
    # if stock_code.startswith(('sh', 'sz')) or stock_code.startswith(('SH', 'SZ')):
    #     return stock_code[:2]
    
    if len(stock_code) == 8: # 期权
        if stock_code.startswith('10'):
            return 'SH'
        if stock_code.startswith('90'):
            return 'SZ'
    stock_code = stock_code[-6:]
    if stock_code.startswith(('50', '51', '60', '90', '110', '113', '132', '204')):
        return 'SH'
    if stock_code.startswith(('00', '13', '18', '15', '16', '18', '20', '30', '39','115', '1318')):
        return 'SZ'
    if stock_code.startswith(('5','6', '9', '7')):
        return 'SH'
    return 'SZ'

def get_stock_type(code):
    """
    判断代码是属于那种类型，目前仅支持 ['fund', 'stock']
    :return str 返回code类型, fund 基金 stock 股票
    """
    if code.startswith(('00', '30', '60','688')):
        return 'stock'
    return 'fund'

def get_stock_codes(real_time=False, stock_type=None, with_exchange=False):
    """获取所有股票 ID 到 all_stock_code 目录下
    real_time:是否实时
    stock_type:(fund 基金 stock 股票)
    with_exchange:是否要加上对应的证券市场编码
    """
    if real_time:
        all_stock_codes_url = 'http://www.shdjt.com/js/lib/astock.js'
        grep_stock_format = '~(\w+)`([^`]+)`'
        grep_stock_codes = re.compile(grep_stock_format)
        response = requests.get(all_stock_codes_url)
        # 这里对id去重
        stock_codes = list(set(grep_stock_codes.findall(response.text)))
        with open(stock_code_path(STOCK_CODE_PATH), 'w') as f:
            f.write(json.dumps(dict(stock=stock_codes), ensure_ascii=False))
    else:
        with open(stock_code_path(STOCK_CODE_PATH)) as f:
            stock_codes = json.load(f)['stock']

    if stock_type:
        stock_codes = [
            (stock[0],stock[1]) for stock in stock_codes if stock_type == get_stock_type(stock[0])
        ]

    if with_exchange:
        stock_codes = [(code_add_prefix(code[0]), code[1])
                       for code in stock_codes]

    return stock_codes

def get_codes_from_web():
    import requests
    all_stock_codes_url = 'http://www.shdjt.com/js/lib/astock.js'
    grep_stock_format = '~(\w+)`([^`]+)`'
    grep_stock_codes = re.compile(grep_stock_format)
    stock_info = []
    try:
        response = requests.get(all_stock_codes_url)
        found_info = grep_stock_codes.findall(response.text)
        stock_info = []
        last = 0
        for e in found_info:
            if e[0] == last:continue
            if not (e[0].startswith('60') or
                    e[0].startswith('30') or
                    e[0].startswith('00') or
                    e[0].startswith('51') or
                    e[0].startswith('15')): continue
            stock_info.append({'code': code_add_prefix(e[0]), 'name': e[1]})
            last = e[0]
    except:
        pass
    return stock_info

def get_stock_codes_exclude_ST():
    all_stock_codes = get_stock_codes(real_time=True, stock_type="stock", with_exchange=False)  # 股票开板或者封板状态
    all_stock_codes = {int(s[0]): s[1] for s in all_stock_codes if
                            s[1].lower().find('st') == -1  # 过滤ST
                            and s[1].lower().find('pt') == -1
                            and s[1].lower().find('退') == -1
                            }  # 过滤ST
    return all_stock_codes

def stock_code_path(STOCK_CODE_PATH):
    if getattr(sys, 'frozen', False):
        pathname = STOCK_CODE_PATH
    else:
        pathname = os.path.join(os.path.dirname(__file__), STOCK_CODE_PATH)
    return pathname

# 方便兼容打包exe
def get_path_dirname():
    if getattr(sys, 'frozen', False):
        application_path = os.path.dirname(sys.executable)
    else:
        # 取当前脚本的上级目录
        application_path = os.path.dirname(os.path.dirname(__file__))
    return application_path

# 回测 读取股票成交记录。提取买入股票清单
def get_buy_list_conf():
    path = ["conf","buy_list.xlsx"]
    path = os.sep.join(path)
    work_dir = get_path_dirname()
    full_path = work_dir + os.sep + path
    buy_list = {}
    import xlrd
    # 1、打开文件
    with xlrd.open_workbook(full_path) as f:
        sheet = f.sheet_by_index(0)
        row_length = sheet.nrows  # 获取行长度
        for i in range(1,row_length): # 过滤第一行标题
            data = sheet.row_values(i)
            _date =  int(data[0])
            code = int(data[2])
            dir = data[4]
            if dir != "证券买入":
                continue
            if _date not in buy_list:
                buy_list[_date] = []
            buy_list[_date].append(code)

    return buy_list

def calc_time_diff(beg, end=None):
    b = datetime.datetime.strptime(str(beg), '%Y%m%d%H%M%S%f')
    if end:
        e = datetime.datetime.strptime(str(end), '%Y%m%d%H%M%S%f')
    else:
        e = datetime.datetime.now()
    seconds_diff = (e - b).total_seconds()
    return seconds_diff

def format_datetime(datetime:str):
    length = len(str(datetime))
    if length == 17:
        return "%s-%s-%s %s:%s:%s.%s" % (
             datetime[:4], datetime[4:6], datetime[6:8], datetime[8:10], datetime[10:12], datetime[12:14], datetime[14:]
        )
    elif length == 9:
        return "%s:%s:%s.%s"% (datetime[0:2],datetime[2:4],datetime[4:6],datetime[6:])
    else:
        return

def debug_data_print(data):
    _datetime = str(data['date']) + "%09d" % data['time']
    diff = calc_time_diff(_datetime)
    print('行情时间[%s]本机时间[%s]股票id[%s]与本机时间间隔[%s]'
          % (format_datetime(_datetime),
             datetime.datetime.now(),
             data['code'],
             diff))


def debug_tran_print(data):
    trade_time = str(data['time'])
    _len = len(trade_time)
    if _len !=17:
        _date = datetime.datetime.now().strftime("%Y%m%d")
        trade_time =_date+ "%09d"%data['time']
    diff = calc_time_diff(trade_time)
    print('成交时间[%s]本机时间[%s]股票id[%06d]与本机时间间隔[%s]'
          % (format_datetime("%09s"%data['time']),
             datetime.datetime.now(),
             int(data['code']),
             diff))

    # 　上市公司季报披露时间：
    # 　　1季报：每年4月1日——4月30日。
    # 　　2季报（中报）：每年7月1日——8月30日。
    # 　　3季报： 每年10月1日——10月31日
    # 　　4季报 （年报）：每年1月1日——4月30日。
    #todo: 年报和一季度报最迟披露时间一致，默认返回年报
def getReportDate(dateTime:int,isUseYear = True):
    # isUseYear:True 返回年报日期 Fasle 返回一季度日期
    mmdd = dateTime % 10000
    year = dateTime // 10000
    dateRange = [1031, 830, 430, ]  # 公布日期

    reportDate = [930, 630, 1231]  if isUseYear else [930, 630, 331,]
    for idx, _dateRange in enumerate(dateRange):
        if mmdd > _dateRange:
            if idx == 2:
                year = year - 1 if isUseYear else year
            return year * 10 ** 4 + reportDate[idx]
    return (year - 1) * 10 ** 4 + reportDate[0]

def getTTmReportDates(reportDate):
    """返回一年滚动需要查询的报告期
        上年同季度日期，上年的年报日期，当季报告期

        如果reportDate是年报日期。则直接返回reportDate
    """
    quarterDate = reportDate % 10 ** 4
    if quarterDate == 1231:
        return [reportDate]
    lastYear = reportDate // 10 ** 4 - 1
    quarterDates = list(set([reportDate % 10 ** 4, 1231]))
    reportDates = [lastYear * 10 ** 4 + q for q in quarterDates]
    reportDates.append(reportDate)
    return reportDates

@Decorator.loadData()
def getSymbolsInfo(isRealTime = False,**kwargs):
    """[生成股票代码，名称，和行业]

    Args:
        isRealTime (bool, optional): [如果为True，会增量添加数据]. Defaults to False.

    Returns:
        [type]: [description]
    """
    fileName = kwargs["fileName"]
    # todo：get_stock_codes 获取全市场股票ID.退市的股票有可能获取不到
    stock_codes = get_stock_codes(real_time=isRealTime, stock_type="stock", with_exchange=True)
    cathchedSymbols = []
    
    if  os.path.exists(fileName):
        df = pd.read_excel(fileName, sheet_name="数据源", index_col=0,engine="openpyxl")
        cathchedSymbols = df["symbol"].tolist()

    symbols = [symbol for symbol,name in stock_codes]
    catchSymbols = list(set(symbols)-set(cathchedSymbols)) # 需要增加的股票
    url = "http://f10.eastmoney.com/CompanySurvey/CompanySurveyAjax"
    params = {
        "code":"",
    }
    data = []
    cnt = len(catchSymbols)
    i = 0
    for symbol,name in stock_codes:
        if symbol not in catchSymbols:
            continue
        i+=1
        print(f"{symbol}{name}\t{i}/{cnt}")
    # for symbol,name in [("SZ.300104","乐视")]:
        params["code"] = symbol.replace(".","")
        r = requests.get(url, params=params)
        textData = r.text
        dataJson = demjson.decode(textData)
        if "fxxg" not in dataJson: # 未上市
            continue
        if "ssrq" not in dataJson["fxxg"]: # 未上市
            continue
        listingDate = pd.to_datetime(dataJson["fxxg"]["ssrq"]) if dataJson["fxxg"]["ssrq"].replace("--","") else np.nan
        series = pd.Series({"symbol":symbol,"name":name,"listingDate":listingDate})
        data.append(series)
    if cathchedSymbols:
        df = df.append(pd.DataFrame(data))
    else:
        df = pd.DataFrame(data)
    return df
if __name__ == '__main__':

    # stock_codes = get_stock_codes(real_time=True,stock_type="stock",with_exchange=True)
    # print(stock_codes)

    # # stocks = ["0"+stock_code[0] if get_stock_exchange(stock_code[0])=="sh" else "1"+stock_code[0] for stock_code in stock_codes ]
    # print(stock_codes)
    # print(getReportDate(20120330))
    # print(getReportDate(20120530))
    # print(getReportDate(20120831))
    # print(getReportDate(20121130))
    # print(pow(2.5,1/11))
    # print(getSymbolsInfo(isRealTime=True))
    print(getSymbolsInfo())
# http://f10.eastmoney.com/CompanySurvey/Index?type=web&code=SZ300104


import os
import struct
import lzma

PACK_TYPE_BYTE = 2
PACK_TYPE_SHORT = 3
PACK_TYPE_INT = 4
PACK_TYPE_LONG = 5
PACK_TYPE_FLOAT = 6
PACK_TYPE_DOUBLE = 7
PACK_TYPE_STRING = 8
PACK_TYPE_BYTES = 9

class stream_reader(object):
    def __init__(self, data):
        self.index = 0
        self.data = data

    def isEnd(self):
        return self.index >= len(self.data)

    def read(self, type, length=0):
        value = None
        if type == PACK_TYPE_BYTE:
            value, = struct.unpack("<B", self.data[self.index:self.index+1])
            self.index += 1
        elif type == PACK_TYPE_SHORT:
            value, = struct.unpack("<H", self.data[self.index:self.index+2])
            self.index += 2
        elif type == PACK_TYPE_LONG:
            value, = struct.unpack("<q", self.data[self.index:self.index+8])
            self.index += 8
        elif type == PACK_TYPE_INT:
            value, = struct.unpack("<I", self.data[self.index:self.index+4])
            self.index += 4
        elif type == PACK_TYPE_FLOAT:
            value, = struct.unpack("<f", self.data[self.index:self.index+4])
            self.index += 4
        elif type == PACK_TYPE_DOUBLE:
            value, = struct.unpack("<d", self.data[self.index:self.index+8])
            self.index += 8
        elif type == PACK_TYPE_STRING:
            value, = struct.unpack("<%ds" % length, self.data[self.index:self.index+length])
            self.index += length
        elif type == PACK_TYPE_BYTES:
            length, = struct.unpack("<H", self.data[self.index:self.index+2])
            self.index += 2
            value, = struct.unpack("<%ds" % length, self.data[self.index:self.index+length])

        return value

def read_shtran_file(c):
    s = stream_reader(c)
    while not s.isEnd():
        data = {}
        data["tradeChannel"] = s.read(PACK_TYPE_INT)
        data["rec_id"] =  s.read(PACK_TYPE_INT)
        data["buy_rec_id"] = s.read(PACK_TYPE_LONG)
        data["sel_rec_id"] = s.read(PACK_TYPE_LONG)
        data["code"] = s.read(PACK_TYPE_INT)
        data["trade_time"] = s.read(PACK_TYPE_INT)
        data["trade_price"] = s.read(PACK_TYPE_DOUBLE)
        data["trade_volume"] = s.read(PACK_TYPE_INT)
        data["buySellFlag"] = s.read(PACK_TYPE_BYTE)
        data["res"] = s.read(PACK_TYPE_BYTE)
        yield data

def read_tran_file(c):
    s = stream_reader(c)
    head = s.read(PACK_TYPE_STRING,4)
    v1 = s.read(PACK_TYPE_BYTE)
    v2 = s.read(PACK_TYPE_BYTE)
    while not s.isEnd():
        data = {}
        data["code"] = s.read(PACK_TYPE_INT)
        data["set_id"] =  s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_INT)
        data["JT_time"] = s.read(PACK_TYPE_INT)
        data["rec_id"] =  s.read(PACK_TYPE_LONG)
        data["buy_order_id"] = s.read(PACK_TYPE_LONG)
        data["sel_order_id"] = s.read(PACK_TYPE_LONG)
        data["trade_price"] = s.read(PACK_TYPE_DOUBLE)
        data["trade_volume"] = s.read(PACK_TYPE_DOUBLE)
        data["trade_type"] = s.read(PACK_TYPE_BYTE)
        data["res1"] = s.read(PACK_TYPE_BYTE)
        data["res2"] = s.read(PACK_TYPE_BYTE)
        data["res3"] = s.read(PACK_TYPE_BYTE)
        yield data


def read_order_file(c):
    s = stream_reader(c)
    head = s.read(PACK_TYPE_STRING,4)
    v1 = s.read(PACK_TYPE_BYTE)
    v2 = s.read(PACK_TYPE_BYTE)
    while not s.isEnd():
        data = {}
        data["code"] = s.read(PACK_TYPE_INT)
        data["set_id"] =  s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_INT)
        data["JT_time"] = s.read(PACK_TYPE_INT)
        data["rec_id"] =  s.read(PACK_TYPE_LONG)
        data["order_price"] = s.read(PACK_TYPE_DOUBLE)
        data["order_volume"] = s.read(PACK_TYPE_DOUBLE)
        data["order_code"] = s.read(PACK_TYPE_BYTE)
        data["reverse1"] = s.read(PACK_TYPE_BYTE)
        data["order_type"] = s.read(PACK_TYPE_BYTE)
        data["reverse2"] = s.read(PACK_TYPE_BYTE)

        yield data
def read_index_file(c):
    s = stream_reader(c)
    head = s.read(PACK_TYPE_STRING,4)
    v1 = s.read(PACK_TYPE_BYTE)
    v2 = s.read(PACK_TYPE_BYTE)

    while not s.isEnd():
        data = {}
        data["code"] = s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_INT)
        data["JT_time"] = s.read(PACK_TYPE_INT)
        data["res"] =  s.read(PACK_TYPE_INT)
        data["last_price"] = round(s.read(PACK_TYPE_DOUBLE),2)
        data["open_price"] = round(s.read(PACK_TYPE_DOUBLE),2)
        data["high_price"] = round(s.read(PACK_TYPE_DOUBLE),2)
        data["low_price"] = round(s.read(PACK_TYPE_DOUBLE),2)
        data["pre_close_price"] = round(s.read(PACK_TYPE_DOUBLE),2)
        data["trade_volume"] = s.read(PACK_TYPE_DOUBLE)

        yield data


def read_hquo_file(c):
    s = stream_reader(c)
    head = s.read(PACK_TYPE_STRING,4)
    v1 = s.read(PACK_TYPE_BYTE)
    v2 = s.read(PACK_TYPE_BYTE)

    while not s.isEnd():
        data = {}
        data["code"] = s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_LONG)
        data["last_price"] =  s.read(PACK_TYPE_DOUBLE)
        data["order_book"] =  s.read(PACK_TYPE_DOUBLE)
        data["trade_volume"] = s.read(PACK_TYPE_DOUBLE)
        data["withdraw_volume"] = s.read(PACK_TYPE_DOUBLE)

        yield data

def read_shorder_file(c):
    s = stream_reader(c)
    while not s.isEnd():
        data = {}
        data["rec_id"] =  s.read(PACK_TYPE_LONG)
        data["code"] = s.read(PACK_TYPE_INT)
        data["order_time"] = s.read(PACK_TYPE_INT)
        data["order_price"] = s.read(PACK_TYPE_DOUBLE)
        data["order_volume"] = s.read(PACK_TYPE_INT)
        data["order_code"] = s.read(PACK_TYPE_BYTE)
        data["reverse1"] = s.read(PACK_TYPE_BYTE)

        yield data

def read_quo_1_5(s):
    while not s.isEnd():
        data = {}
        data["symbol"] =  s.read(PACK_TYPE_INT)
        data["date"] =  s.read(PACK_TYPE_INT)
        data["jt_time"] = s.read(PACK_TYPE_INT)
        data["local_time"] = s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_INT)
        data["last_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["open_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["high_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["low_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["pre_close_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        for i in range(1, 11):
            data["buy_price%02d"%i] = round(s.read(PACK_TYPE_FLOAT),2)
        for i in range(1, 11):
            data["buy_volume%02d"%i] = s.read(PACK_TYPE_INT)
        for i in range(1, 11):
            data["sell_price%02d"%i] = round(s.read(PACK_TYPE_FLOAT),2)
        for i in range(1, 11):
            data["sell_volume%02d"%i] = s.read(PACK_TYPE_INT)
        data["uplimit"] = s.read(PACK_TYPE_FLOAT)
        data["downlimit"] = s.read(PACK_TYPE_FLOAT)
        data["total_amount"] = s.read(PACK_TYPE_DOUBLE)
        data["sp"] = s.read(PACK_TYPE_STRING, 8)
        data["ts"] = s.read(PACK_TYPE_STRING, 8)
        yield data

def read_quo_1_4(s):
    while not s.isEnd():
        data = {}
        data["symbol"] =  s.read(PACK_TYPE_INT)
        data["date"] =  s.read(PACK_TYPE_INT)
        data["jt_time"] = s.read(PACK_TYPE_INT)
        data["local_time"] = s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_INT)
        data["last_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["open_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["high_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["low_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        data["pre_close_price"] = round(s.read(PACK_TYPE_FLOAT),2)
        for i in range(1, 11):
            data["buy_price%02d"%i] = round(s.read(PACK_TYPE_FLOAT),2)
        for i in range(1, 11):
            data["buy_volume%02d"%i] = s.read(PACK_TYPE_INT)
        for i in range(1, 11):
            data["sell_price%02d"%i] = round(s.read(PACK_TYPE_FLOAT),2)
        for i in range(1, 11):
            data["sell_volume%02d"%i] = s.read(PACK_TYPE_INT)
        data["uplimit"] = s.read(PACK_TYPE_FLOAT)
        data["downlimit"] = s.read(PACK_TYPE_FLOAT)
        data["sp"] = s.read(PACK_TYPE_STRING, 8)
        data["ts"] = s.read(PACK_TYPE_STRING, 8)
        yield data

def read_quo_1_1(s):
    while not s.isEnd():
        data = {}
        data["symbol"] =  s.read(PACK_TYPE_INT)
        data["date"] =  s.read(PACK_TYPE_INT)
        data["jt_time"] = s.read(PACK_TYPE_INT)
        data["local_time"] = s.read(PACK_TYPE_INT)
        data["time"] = s.read(PACK_TYPE_INT)
        data["last_price"] = s.read(PACK_TYPE_DOUBLE)
        data["open_price"] = s.read(PACK_TYPE_DOUBLE)
        data["high_price"] = s.read(PACK_TYPE_DOUBLE)
        data["low_price"] = s.read(PACK_TYPE_DOUBLE)
        data["pre_close_price"] = s.read(PACK_TYPE_DOUBLE)
        data["buy_price01"] = s.read(PACK_TYPE_DOUBLE)
        data["buy_volume01"] = s.read(PACK_TYPE_LONG)
        data["sell_price01"] = s.read(PACK_TYPE_DOUBLE)
        data["sell_volume01"] = s.read(PACK_TYPE_LONG)
        data["uplimit"] = s.read(PACK_TYPE_DOUBLE)
        data["downlimit"] = s.read(PACK_TYPE_DOUBLE)
        data["sp"] = s.read(PACK_TYPE_STRING, 8)
        data["ts"] = s.read(PACK_TYPE_STRING, 8)
        yield data

def read_quo_file(c):
    s = stream_reader(c)
    head = s.read(PACK_TYPE_STRING,4)
    v1 = s.read(PACK_TYPE_BYTE)
    v2 = s.read(PACK_TYPE_BYTE)

    if v1 == 1 and v2 == 1:
        return read_quo_1_1(s)
    elif v1 == 1 and v2 == 4:
        return read_quo_1_4(s)
    elif v1 == 1 and v2 == 5:
        return read_quo_1_5(s)


def parse_file(path, is_compress):
    p = path[:path.rfind(".")]
    ext = path[path.rfind(".")+1:]
    if ext == "csv": return

    zf = open(path, "rb")
    c = zf.read()
    if is_compress:
        c = lzma.decompress(c)
    zf.close()

    of = "%s_%s.log" % (p, ext)
    if ext == "tran":
        title = ["股票编码", "成交时间", "记录号", "频道代码","", "成交价格",
                 "成交数量","","", "市场", "买方id", "卖方id","成交类型(52：撤单，70：成交)",
                 ]
        with open(of, "wt") as f:
            line = (",").join(title)
            f.write(line + "\n")
            for d in read_tran_file(c):
                linee = []
                linee.append("%06d" % d["code"])
                linee.append(d["time"])
                linee.append(d["JT_time"])
                linee.append(d["rec_id"])
                linee.append(d["set_id"])
                linee.append(0)# tradeChannel
                linee.append(d["trade_price"])
                linee.append(d["trade_volume"])
                linee.append(0)# tradeAmount
                linee.append(0)# UNIX
                linee.append("SZ")
                linee.append(d["buy_order_id"])
                linee.append(d["sel_order_id"])
                linee.append(d["trade_type"])
                linee = [str(s) for s in linee]
                line = (",").join(linee)
                f.write(line+"\n")
    elif ext== "order":
        title =["股票编码","委托时间","记录号","频道代码","委托价格",
                "委托数量","市场","买卖方向(1:买2:卖)","订单类别(1:市价2:限价)",
                ]
        with open(of, "wt") as f:
            line = (",").join(title)
            f.write(line + "\n")
            for d in read_order_file(c):
                linee = []
                linee.append("%06d" % d["code"])
                linee.append(d["time"])
                linee.append(d["JT_time"])
                linee.append(d["rec_id"])
                linee.append(d["set_id"])
                linee.append(d["order_price"])
                linee.append(d["order_volume"])
                linee.append("SZ")
                linee.append(d["order_code"])
                linee.append(d["order_type"])
                linee = [str(s) for s in linee]
                line = (",").join(linee)
                f.write(line+"\n")
    elif ext == "shorder":

        with open(of, "wt") as f:
            for d in read_shorder_file(c):
                linee = []
                linee.append("%06d" % d["code"])
                linee.append(d["rec_id"])
                linee.append(d["order_time"])
                linee.append(d["order_price"])
                linee.append(d["order_volume"])
                linee.append(d["order_code"])
                linee = [str(s) for s in linee]
                line = (",").join(linee)
                f.write(line+"\n")
    elif ext == "shtran":
        title =["股票编码","日期","成交","本地时间","交易所生成时间",
                "最新价","开盘价","最高价","最低价","上一个交易日收盘价",
                "买一价","买一量","卖一价","卖一量","涨停价",
                "跌停价","成交量","SecurityPhaseTag","TradeStatus",
                ]
        with open(of, "wt") as f:
            line = (",").join(title)
            f.write(line + "\n")
            for d in read_shtran_file(c):
                linee = []
                linee.append("%06d" % d["code"])
                linee.append(d["tradeChannel"])
                linee.append(d["rec_id"])
                linee.append(d["buy_rec_id"])
                linee.append(d["sel_rec_id"])
                linee.append(d["trade_time"])
                linee.append(d["trade_price"])
                linee.append(d["trade_volume"])
                linee.append(d["buySellFlag"])
                linee = [str(s) for s in linee]
                line = (",").join(linee)
                f.write(line+"\n")
    elif ext == "quo":
        title =["股票编码","日期","津泰时间","本地时间","交易所生成时间",
                "最新价","开盘价","最高价","最低价","上一个交易日收盘价",
                "买一价","买一量","买二价","买二量","卖一价","卖一量","卖二价","卖二量","涨停价",
                "跌停价","成交量","SecurityPhaseTag","TradeStatus",
                ]
        with open(of, "wt") as f:
            line = (",").join(title)
            f.write(line + "\n")
            for d in read_quo_file(c):
                linee = []
                linee.append("%06d" % d["symbol"])
                linee.append(d["date"])
                linee.append(d["jt_time"])
                linee.append(d["local_time"])
                linee.append(d["time"])
                linee.append(d["last_price"])
                linee.append(d["open_price"])
                linee.append(d["high_price"])
                linee.append(d["low_price"])
                linee.append(d["pre_close_price"])
                linee.append(d["buy_price01"])
                linee.append(d["buy_volume01"])
                linee.append(d["buy_price02"])
                linee.append(d["buy_volume02"])
                linee.append(d["sell_price01"])
                linee.append(d["sell_volume01"])
                linee.append(d["sell_price02"])
                linee.append(d["sell_volume02"])
                linee.append(d["uplimit"])
                linee.append(d["downlimit"])
                if "total_amount" in d:
                    linee.append(d["total_amount"])
                linee.append(d["sp"])
                linee.append(d["ts"])

                linee = [str(s) for s in linee]
                line = (",").join(linee)
                f.write(line+"\n")
    elif ext == "idx":
        title =["股票编码","交易时间","津泰时间",
                "最新价","开盘价","最高价","最低价","上一个交易日收盘价",
                "成交量"
                ]
        with open(of, "wt") as f:
            line = (",").join(title)
            f.write(line + "\n")
            for d in read_index_file(c):
                linee = []
                linee.append("%06d" % d["code"])
                linee.append(d["JT_time"])
                linee.append(d["time"])
                linee.append(d["last_price"])
                linee.append(d["open_price"])
                linee.append(d["high_price"])
                linee.append(d["low_price"])
                linee.append(d["pre_close_price"])
                linee.append(d["trade_volume"])

                linee = [str(s) for s in linee]
                line = (",").join(linee)
                f.write(line + "\n")
    elif ext == "hquo":
        title =["股票编码","生成时间",
                "最新价","成交量","封单量",
                "撤单量"
                ]
        with open(of, "wt") as f:
            line = ("\t").join(title)
            f.write(line + "\n")
            for d in read_hquo_file(c):
                linee = []
                linee.append("%06d" % d["code"])
                linee.append(d["time"])
                linee.append(d["last_price"])
                linee.append(d["trade_volume"])
                linee.append(d["order_book"])
                linee.append(d["withdraw_volume"])
                linee = [str(s) for s in linee]
                line = ("\t").join(linee)
                f.write(line+"\n")


if __name__ == '__main__':
    rootdir = r'F:\hqtest\data1'
    is_compress = False

    list = os.listdir(rootdir) #列出文件夹下所有的目录与文件
    for i in range(0,len(list)):
        path = os.path.join(rootdir,list[i])
        if os.path.isfile(path):
            parse_file(path, is_compress)


    # parse_file(r"000002.quo", False)



# -*- coding: utf-8 -*-
# 对策略的回测结果分析
import pandas as pd
import numpy as np
import json
import os
import re
import sys
import requests
import datetime
from  tmqc.common import basefunc
from tmqc.frame import data_center
from tradeTools import DataMgr
from tradeTools import Decorator


CON_NET_VALUE_FILENAME = "stock_day_net_value.log" # 要读取的日志文件名
STRATEGY_PATH_CONF = [os.path.dirname(os.path.dirname(__file__)),"FundFlow"]
STRATEGY_PATH_CONF.append("20150713INDEX_SYMBOL[SH.000905]USE_PRICEclose是否首次突破才买False是否全平[True]是否平均持仓[True]考虑涨跌停True可持仓数量20")
# STRATEGY_PATH_CONF = [os.path.dirname(os.path.dirname(__file__)),"reversal","log_500市值加权"]
# pathCnf = [STRATEGY_PATH, f"{CON_NET_VALUE_FILENAME}.log"]
path = os.sep.join(STRATEGY_PATH_CONF)
full_path = path + os.sep + CON_NET_VALUE_FILENAME
# print(full_path)



class Mgr(DataMgr.Mgr):
    @Decorator.loadData()
    def addIndex(self,frequency = 1):
        df = self.load()
        begTime = df.iloc[0].name # 策略开始时间
        dfs = [df]
        for indexSymbol in ["SH.000300","SH.000905","SH.000001"]:
            idx = self.genIDXData(indexSymbol=indexSymbol, frequency=frequency)
            idx = idx[idx.index >= str(begTime)]
            dfs.append(idx[[indexSymbol]])
        total = pd.concat(dfs, axis=1, join='outer', )
        return total
    
    # df 每日收益率
    @Decorator.loadData()
    def qryAnalyse(self,df,strategy,frequency):
        begTime = df.iloc[0].name # 策略开始时间
        dfs = [df]
        for indexSymbol in ["SH.000300","SH.000905","SH.000001"]:
            idx = self.genIDXData(indexSymbol=indexSymbol, frequency=frequency)
            idx = idx[idx.index >= str(begTime)]
            dfs.append(idx[[indexSymbol]])
        total = pd.concat(dfs, axis=1, join='outer', )

        d = {}
        for name, s in total.items():
            tradeDays = s.size
            # todo:股票k线数据丢失或停牌，会导致收益率均值为0.加权为空，影响占用天数的统计
            keepTradeDays = tradeDays - s.isnull().sum() # 占用天数
            std = s[~s.isnull()].std()
            yearStd = std * pow(252, 0.5)  # 年化标准差

            avg = s[~s.isnull()].mean()
            std = s[s < avg].std()  # 每日收益率标准差
            SDRyearStd = 2* std * pow(252, 0.5)  # #补偿只有基准以下的收益率计算入标准差，需要再将标准差乘以2
            d[name] = [tradeDays,keepTradeDays,yearStd,SDRyearStd]
        ret = pd.DataFrame(d,index=["总天数","实际占用天数","年化标准差","SDR年化标准差"]).T
        total.fillna(0, inplace=True)  # 停牌的日期。收益率为设为0
        totalCumprod = (1 + total).cumprod()
        ret["累计净值"] = totalCumprod.iloc[-1,:].T

        ret["复合年收益率"] = pow(ret["累计净值"],252/ret["总天数"])-1
        ret["夏普率"] = (ret["复合年收益率"]-0.02)/ret["年化标准差"]
        ret["SDR夏普率"] = (ret["复合年收益率"]-0.02)/ret["SDR年化标准差"]

        ret["年收益率_根据实际占用天数"] =pow(ret["累计净值"],252/ret["实际占用天数"])-1
        ret["夏普率_根据实际占用天数"] = (ret["年收益率_根据实际占用天数"]-0.02)/ret["年化标准差"]
        ret["SDR夏普率_根据实际占用天数"] = (ret["年收益率_根据实际占用天数"] - 0.02) / ret["SDR年化标准差"]

        maximumDrawdown = 1-totalCumprod/np.maximum.accumulate(totalCumprod)
        ret["最大回撤"] = maximumDrawdown.max()
        ret["最大回撤日期"] = maximumDrawdown.idxmax()
        return ret
    
    @Decorator.loadData()
    def qryRateByYear(self,df,frequency,indexSymbol,strategyName):
        idx = self.genIDXData(indexSymbol=indexSymbol, frequency=frequency)
        begTime = df.iloc[0].name # 策略开始时间
        endTime = df.iloc[-1].name # 策略结束时间
        idx = idx[(idx.index >= begTime)&(idx.index <= endTime)]
        total = pd.concat([df,idx[[indexSymbol]]], axis=1, join='outer', )
        s = []
        for i ,y  in total.resample("Y"):
            _y = (1 + y).cumprod()
            _s = _y.iloc[-1]-1
            _s.name = i.year
            s.append(_s)
        r = pd.DataFrame(s)
        return r
    
    @Decorator.loadData()
    def qryDayNetValue(self,logName):
        STRATEGY_PATH_CONF = [os.path.dirname(os.path.dirname(__file__)),"FundFlow",logName]
        # LOG_FILE_NAME = [year for year in range(2010,2012)]
        CON_NET_VALUE_FILENAME = "stock_day_net_value {year}.log"  # 要读取的日志文件名
        dfs = []
        for year in range(2010, 2022):
            paths = []
            paths.extend(STRATEGY_PATH_CONF)
            paths.append(str(year))
            paths.append(CON_NET_VALUE_FILENAME.format(year = year))

            path = os.sep.join(paths)
            print(path)
            df = pd.read_csv(path, engine='python', encoding='gb2312', sep='\t',header=None)
            print(df)
            dfs.append(df)
        total  = pd.concat(dfs, axis=0, join='outer', )
        total.columns = ["日期","净值","回撤","上证指数","上证回撤","沪深300","沪深300回撤","上证50","上证50回撤","资产","利润","保证金","手续费","滑点损失"]

        total["每日收益率"] = total["净值"]/total["净值"].shift() -1
        return total
    #     print(max(1-totalCumprod["SH.000300"]/np.maximum.accumulate(totalCumprod["SH.000300"])))
    #     print(idmax(1-totalCumprod["SH.000300"]/np.maximum.accumulate(totalCumprod["SH.000300"])))
    # # def __init__(self):
    #     self.oldDc = data_center.use()
        # self.df = self.load()

    def load(self):
        df = pd.read_csv(full_path,engine='python', encoding='gb2312', sep='\t')
        df['date'] = pd.to_datetime(df['日期'], format='%Y%m%d')
        df.set_index('date', inplace=True)
        df.drop_duplicates( keep='first',inplace=True)
        return df
    #
    # def concat(self):
    #     indexSymbol = "SH.000905"
    #     df = pd.concat([self.df,self.genIDXData()],axis = 1,join="inner")
    #     dateIdx = df[df['净值']!= 1].iloc[0].name # 首次开仓日期
    #     df = df[df.index>=dateIdx].copy()
    #     df["中证500"] = (1 + df[f"{indexSymbol}收益率"]).cumprod()
    #     name = path+os.sep+f"{STRATEGY_PATH_CONF[-1]}.xlsx"
    #     df.to_excel(name, sheet_name="数据源")
# def genYearReport():
#
#     df = df.resample('Y').last()
#     # Index(['日期', '净值', '回撤', '上证指数', '上证回撤', '沪深300', '沪深300回撤', '上证50', '上证50回撤',
#     #        '资产', '利润', '保证金', '手续费', '滑点损失'],
#     #       dtype='object')
#     df.to_excel("年度净值报告 .xlsx", sheet_name="数据源")

if __name__ == '__main__':
    mgr = Mgr()
    CON_FREQUENCY = 1
    CON_INDEX_SYMBOL = "SH.000300"
    # idxDf = mgr.genIDXData(symbol=CON_INDEX_SYMBOL, fileExtension=CON_FREQUENCY)
    # df = mgr.qryDayNetValue(logName="20100424INDEX_SYMBOL[SH.000905]USE_PRICEopen是否首次突破才买False是否全平[True]是否平均持仓[False]考虑涨跌停True可持仓数量99999")
    # mgr.concat()
    print(mgr.qryRateByYear())



import abc
import re
from common import Decorator
from collections import namedtuple


class ConfigBase:
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def __init__(self):
        pass

    @abc.abstractmethod
    def get_mutil(self, code):
        return 0

    @abc.abstractmethod
    def get_open_fee(self, code):
        return 0

    @abc.abstractmethod
    def get_close_fee(self, code):
        return 0

    @abc.abstractmethod
    def get_close_today_fee(self, code):
        return 0

    @abc.abstractmethod
    def get_margin_rate(self, code):
        return 0

    @abc.abstractmethod
    def get_slippage(self, code):
        return 0.000

    @abc.abstractmethod
    def init(self, datacenter):
        pass


@Decorator.singleton
class cStock(ConfigBase):
    def __init__(self):
        super().__init__()

    def get_mutil(self, code):
        return 1

    def get_open_fee(self, code):
        """返回率，值"""
        return 0.0002, 0

    def get_close_fee(self, code):
        """返回率，值"""
        # return 0.0002, 0
        return 0.0002, 0

    def get_close_today_fee(self, code):
        """返回率，值"""
        # return 0.0002, 0
        return 0.0000, 0

    def get_margin_rate(self, code):
        return 1
    
    def get_slippage(self, code):
        return 0.000    

    def get_slippage(self, code):
        return 0.001
    
    def init(self, datacenter):
        pass


@Decorator.singleton
class cFurture(ConfigBase):
    def __init__(self):
        super().__init__()
        self.com_mar = {}
        self.pt_mul = {}
        self.slippage ={}

        ContractFee = namedtuple('ContractFee', ['open_fee', 'close_fee', 'close_today_fee', 'margin_rate'])
        with open("conf/contract_com_mar.ini", "rt") as f:
            l = f.readline()
            while l:
                arrs = re.split(r"\s+", l)
                if arrs[0][0] != "#":
                    self.com_mar[arrs[0]] = ContractFee(open_fee=float(arrs[1]),
                                                        close_fee=float(arrs[2]),
                                                        close_today_fee=float(arrs[3]),
                                                        margin_rate=float(arrs[4]))
                l = f.readline()

        ContractMutil = namedtuple('ContractMutil', ['mutil', 'mintick'])
        with open("conf/contract_pt_mul.ini", "rt") as f:
            l = f.readline()
            while l:
                arrs = re.split(r"\s+", l)
                if arrs[0][0] != "#":
                    self.pt_mul[arrs[0]] = ContractMutil(mintick=float(arrs[1])/10000.0, mutil=float(arrs[2]))
                l = f.readline()
                
        ContractSlippage = namedtuple('ContractSlippage', ['slippage'])
        with open("conf/contract_slippage.ini", "rt") as f:
            l = f.readline()
            while l:
                arrs = re.split(r"\s+", l)
                if arrs[0][0] != "#":
                    self.slippage[arrs[0]] = ContractSlippage(slippage=float(arrs[1])/100)
                l = f.readline()
                
    def get_mutil(self, code):
        if code[-2:] == 'L0': code = code[:-2]
        code = ''.join([i for i in code if not i.isdigit()])
        c = self.pt_mul.get(code)
        if c is None:
            return 1
        return c.mutil

    def get_open_fee(self, code):
        if code[-2:] == 'L0': code = code[:-2]
        code = ''.join([i for i in code if not i.isdigit()])
        c = self.com_mar.get(code)
        if c is None:
            return 0,0
        if c.open_fee > 1e7:
            return 0, c.open_fee / 1e7
        return c.open_fee / 1e7, 0

    def get_close_fee(self, code):
        if code[-2:] == 'L0': code = code[:-2]
        code = ''.join([i for i in code if not i.isdigit()])
        c = self.com_mar.get(code)
        if c is None:
            return 0,0
        if c.close_fee > 1e7:
            return 0, c.close_fee / 1e7
        return c.close_fee / 1e7, 0

    def get_close_today_fee(self, code):
        if code[-2:] == 'L0': code = code[:-2]
        code = ''.join([i for i in code if not i.isdigit()])
        c = self.com_mar.get(code)
        if c is None:
            return 0,0
        if c.close_today_fee > 1e7:
            return 0, c.close_today_fee / 1e7
        return c.close_today_fee / 1e7, 0

    def get_margin_rate(self, code):
        if code[-2:] == 'L0': code = code[:-2]
        code = ''.join([i for i in code if not i.isdigit()])
        c = self.com_mar.get(code)
        if c is None:
            return 1
        return c.margin_rate / 1e3
    
    def get_slippage(self, code):
        return 0.000    

    def get_slippage(self, code):
        if code[-2:] == 'L0': code = code[:-2]
        code = ''.join([i for i in code if not i.isdigit()])
        c1 = self.slippage.get(code)
        if c1 is None:
            return 1
        
        c2 = self.pt_mul.get(code)
        if c2 is None:
            return 1
        
        return c1.slippage * c2.mintick

    def init(self, datacenter):
        pass


@Decorator.singleton
class cOption(ConfigBase):
    def __init__(self):
        super().__init__()
        self.info = None
        self.mutils = {}

    def get_mutil(self, code):
        mutil = self.info[self.info['code'] == code].iloc[-1]['contract_unit']
        return mutil

    def get_open_fee(self, code):
        """返回率，值"""
        return 0, 2

    def get_close_fee(self, code):
        """返回率，值"""
        return 0, 2

    def get_close_today_fee(self, code):
        """返回率，值"""
        return 0, 2

    def get_margin_rate(self, code):
        return 1

    def get_slippage(self, code):
        return 0.0001

    def init(self, datacenter):
        self.info = datacenter.option.info


stock = cStock()
future = cFurture()
option = cOption()

if __name__ == '__main__':
    import sys, os

    # sys.path.append('..')
    os.chdir('..')

    sc = stock
    fc = future
    oc = option

    print(fc.get_mutil("FG"))
    print(fc.get_open_fee("FG"))
    print(fc.get_close_fee("FG"))



'''
    编译前要进入环境 python3.6
    F:/ProgramFiles/Anaconda3/Scripts/activate
    conda activate F:\ProgramFiles\Anaconda3

    进入目录
    cd cy/py_hq_client

    使用命令 来编译扩展
    python setup.py build_ext --inplace
    
    Cython 语法复杂
    导出定义等相关资料可参见如下网页

    https://cython.readthedocs.io/en/latest/src/userguide/language_basics.html
    https://cython.readthedocs.io/en/latest/src/userguide/extension_types.html
    https://cython.readthedocs.io/en/latest/src/userguide/wrapping_CPlusPlus.html
    https://python3-cookbook.readthedocs.io/zh_CN/latest/c15/p10_wrap_existing_c_code_with_cython.html
    https://www.jianshu.com/p/4a08fe5225a5
    
    C/C++函数定义时 注意不能使用 python 关键字 
    https://www.w3school.com.cn/python/python_ref_keywords.asp
    
    
'''
import glob
import os

from inc.walk_in_file import *
from inc.ms_inc import *

from distutils.core import setup
from distutils.extension import Extension

from Cython.Distutils import build_ext

ext_modules = [
    Extension(
        'py_hq_client',
        ['_hq_client.pyx'],
        include_dirs = ['./lib',],
        libraries=['hq_client'],
        library_dirs=['./lib',],
        runtime_library_dirs=[],
        language="c++",
        # extra_compile_args=['/MD']
    )
]

setup(
    name='py_hq_client',
    version='0.0.1',
    description='python interface of hq_client',
    author='cjxx',
    license='GPL',
    # packages=['py_hq_client'],
    ext_modules = ext_modules,
    cmdclass={'build_ext':build_ext}
)

# 删除cpp
DeleteFile('_hq_client.cpp')
DeleteFile('cfg_hq_client.ini')
DeleteFile('hq_client.dll')

# 拷贝新dll 和配置文件
CopyFileToDst('./lib/cfg_hq_client.ini', './')
CopyFileToDst('./lib/hq_client.dll', './')

# 改名dll
DeleteFile('py_hq_client.pyd')
try_rename('py_hq_client.cp36-win_amd64.pyd', 'py_hq_client.pyd')


# -*- coding: utf-8 -*-
# @Time    : 2018/8/10 14:01
# @Author  : hc
# @Site    : 
# @File    : Decorator.py
# @Software: PyCharm
from functools import wraps
import os
import pandas as pd

def singleton(cls, *args, **kwargs):
    instance = {}  # 创建字典来保存实例

    def get_instance(*args, **kwargs):
        if cls not in instance:  # 若实例不存在则新建
            instance[cls] = cls(*args, **kwargs)
        return instance[cls]

    return get_instance

# 装饰器，优先加载静态文件
# def loadData(*dargs, **dkargs):
def loadData(*dargs, **dkargs):
    def outer(func,*args, **kwargs):
        def inner(*args, **kwargs):
            nodeFileName = func.__name__[3:] # 构造文件夹名称
            excelName = f"{nodeFileName}"
            isRealTime = kwargs['isRealTime'] if "isRealTime" in kwargs else False
            excelNames = []
            for k,v in kwargs.items(): # 参数的键值对作为补充文件名
                if k == "isRealTime" or k=="df":
                    continue
                excelNames.append(k)
            excelNames.sort()
            for k in excelNames:
                excelName += f"_{k}[{str(kwargs[k])}]"
            co_filename = os.path.normcase(func.__code__.co_filename)
            # 定位到工作空间目录 workspace
            _paths = co_filename.split("\\")[0:2]

            _paths.append("data")

            if "path" in dkargs and dkargs["path"] == "data":
                # 装饰器传参 path：设置为"data".则读取data路径。用于可复用的数据
                dataPath =""
            else:
                # 获取函数代码所在的文件名称。用于构造读取静态文件的路径
                # 取函数代码所在文件名中 _ 分割的第二个关键字
                dataPath = co_filename.split("\\")[-1].split(".")[0].split("_")[0:1]
                dataPath = "_".join(dataPath)
                _paths.append(dataPath)
            _paths.append(nodeFileName)
            _paths.append(excelName+".xlsx")
            fileName = os.sep.join(_paths)  #
            path = os.path.dirname(fileName)
            if not os.path.exists(path):os.makedirs(path)
            if not os.path.exists(fileName) or isRealTime:
                print(func.__name__ + f"\033[32m开始生成数据[{fileName}]\033[0m")
                df =  func(fileName = fileName,*args,**kwargs )
                df.to_excel(fileName, sheet_name="数据源")
                return df
            print(func.__name__ + f"读取数据[{fileName}]")
            df = pd.read_excel(fileName, sheet_name="数据源", index_col=0,engine="openpyxl")
            # df = pd.read_excel(fileName, sheet_name="数据源", index_col=0,)
            return df
        return inner
    return outer


# -*- coding: utf-8 -*-
import json
import os
import sys
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0,BASE_DIR)
from common import basefunc
from common.define import SUBSTITUTE_FLAG
from common import log
from tradeTools import helpers
from re import sub

SETTING = basefunc.get_strategy_setting()
FILENAME= "按照清单净值计算_%s"
class ETF_Info:
    def __init__(self,datetime):
        # self.dc = data_center.use()
        self.etf_infos = self.load(datetime=datetime)
        self.etf_list = self.gen_list()

    # def set_last_datetime(self): # 设置上一个交易日
    #     if self.datetime not in self.dc.trade_days:
    #         last_datetime = self.dc.trade_days[-1]
    #         if last_datetime>=self.datetime:
    #             raise Exception('上一个交易日[{last_datetime}]大于当前交易日[{datetime}]'
    #                             .format(last_datetime= last_datetime,datetime = self.datetime))
    #         return last_datetime
    #     return self.dc.trade_days[self.dc.trade_days.index(self.datetime)-1]

    # 根据成分股的数量从高到低排序。然后分组
    def gen_list(self):
        etf_list = []
        for id2 in self.etf_infos.keys():
            constituent_stocks = self.get_constituent_stock_info(id2)
            cnt = len(constituent_stocks)
            etf_list.append((cnt,id2))
        etf_list.sort(reverse=True)
        res =[[] for i in range(SETTING.GROUP_NUM)]

        for _idx,_ in enumerate(etf_list):
            id2 = _[1]
            idx =_idx%SETTING.GROUP_NUM if _idx//SETTING.GROUP_NUM%2 == 0 else -(_idx %SETTING.GROUP_NUM) - 1
            # idx = _idx % SETTING.GROUP_NUM
            res[idx].append(id2)
        return res

    def load(self,datetime:int):
        self.datetime = datetime
        # self.last_datetime = self.set_last_datetime()

        etf_infos = {}
        path = os.sep.join([basefunc.get_path_dirname(), "conf", "etf",str(datetime // 100)])
        if not os.path.exists(path):
            txt = "不存在该路径{}".format(path)
            print(txt)
            return etf_infos
        full_paths = [os.sep.join([path, "etf_%s_%s.conf" % (market_name,datetime)]) for market_name in ["sh","sz"]]

        for full_path in full_paths:
            if not os.path.exists(full_path):
                print("未找到etf配置路径 ",full_path)
                continue
            with open(full_path) as f:
                obj = json.load(f)
                for fundid2,details in obj.items():
                    constituent_stock_info = {int(stock_id):v for stock_id,v in details["constituent_stock_info"].items()}
                    details["constituent_stock_info"] = constituent_stock_info
                    etf_infos.update({helpers.code_add_prefix(fundid2):details})
            print("读取etf配置路径 ",full_path)
        return etf_infos

    def get_creationRedemptionUnit(self,fundid2)->int:
        """申赎单位(份)"""
        return int(self.etf_infos[fundid2]["creationRedemptionUnit"])

    def get_estimatedCashComponent(self,fundid2)->float:
        """预估现金"""
        # estimatedCashComponent = float(sub(r'[^-?\d.]', '', ))
        return self.etf_infos[fundid2]["estimatedCashComponent"]

    def get_constituent_stock_info(self,fundid2)->dict:
        """成分股"""
        d = {helpers.code_add_prefix(code):info for code,info in self.etf_infos[fundid2]["constituent_stock_info"].items()}
        return d

    def get_idx(self,fundid2)->int:
        """跟踪指数"""
        return int(self.etf_infos[fundid2]["index_id"])
    def getRedemptionUnitAssetValue(self,fundid2)->float:
        """上一个交易日的单位资产净值"""
        return float(self.etf_infos[fundid2]["fRedemptionUnitAssetValue"])

    def get_market_type(self,fundid2)->int: # 十位：（1:沪市 2:深市） 个位（2;跨市）
        """市场类型标志"""
        return int(self.etf_infos[fundid2]["market_type"])

    # def calc_estimatedCashComponent(self,fundid2,fRedemptionUnitAssetValue): # 计算预估现金
    #     """
    #     T日预估现金部分
    #     ＝T-1日最小申购、赎回单位的基金资产净值
    #     －（申购、赎回清单中必须用现金替代的固定替代金额
    #     ＋申购、赎回清单中可以用现金替代成份证券的数量与T日预计开盘价相乘之和
    #     +申购、赎回清单中禁止用现金替代成份证券的数量与T日预计开盘价相乘之和）
    #     """
    #     # 成分股价值
    #     log.WriteLog(FILENAME%self.datetime,"code\t股数\t%s日收盘价\t价值"%self.last_datetime)
    #     sum = 0
    #     for code, v in self.get_constituent_stock_info(fundid2).items():
    #         if v["substituteFlag"] == SUBSTITUTE_FLAG.MUST.value:
    #             sum += v["cashAmount"]
    #             log.WriteLog(FILENAME % self.datetime, f"{code}\t\t\t{v['cashAmount']}")
    #             continue
    #         stock = helpers.code_add_prefix(code)
    #         close, datetime = self.dc.get_nearest_price(stock, self.last_datetime)
    #         if datetime != self.last_datetime:
    #             print("%s 有停牌" % stock)
    #         if self.dc.query_dividend_by_reg_date(stock,datetime):
    #             print("%s 股权登记" % stock)
    #         quantity = v["quantity"]
    #         sum+=close*quantity
    #         log.WriteLog(FILENAME % self.datetime, f"{code}\t{quantity}\t{close}\t{close*quantity}")
    #
    #     estimatedCashComponent = fRedemptionUnitAssetValue - sum
    #     web_estimatedCashComponent = self.get_estimatedCashComponent(fundid2)
    #     log.WriteLog(FILENAME % self.datetime, f"根据股数计算的净值\t{fRedemptionUnitAssetValue}\t-清单净值\t{sum}\t=预估现金{estimatedCashComponent}公告的预估现金{web_estimatedCashComponent}")
    #     return estimatedCashComponent,web_estimatedCashComponent

if __name__ == '__main__':
    etf = ETF_Info(20210621)
    fundid2 ="SZ.159801"
    # print(etf.etf_infos[fundid2])
    # a = etf.calc_estimatedCashComponent(fundid2)
    for i in etf.etf_list:
        print(i)
    # # etf.calc_estimatedCashComponent(fundid2)
    # # print(type(etf.get_estimatedCashComponent(fundid2)))
    # for k,v in etf.get_constituent_stock_info(fundid2).items():
    #     print(k,v)
    # print(etf.get_estimatedCashComponent(fundid2))
    # print(etf.get_estimatedCashComponent(fundid2))
    # print(etf.etf_infos.keys())
    # print(etf.constituent_stock_info)


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from tradeTools import indexData_pb2 as indexData__pb2


class indexDataStub(object):
  """The greeting service definition.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.sayHello = channel.unary_unary(
        '/indexData.indexData/sayHello',
        request_serializer=indexData__pb2.JSONRequest.SerializeToString,
        response_deserializer=indexData__pb2.JSONResponse.FromString,
        )
    self.getSymbols = channel.unary_unary(
        '/indexData.indexData/getSymbols',
        request_serializer=indexData__pb2.JSONRequest.SerializeToString,
        response_deserializer=indexData__pb2.Symbols.FromString,
        )


class indexDataServicer(object):
  """The greeting service definition.
  """

  def sayHello(self, request, context):
    """Sends a greeting
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def getSymbols(self, request, context):
    """获取开仓信号
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_indexDataServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'sayHello': grpc.unary_unary_rpc_method_handler(
          servicer.sayHello,
          request_deserializer=indexData__pb2.JSONRequest.FromString,
          response_serializer=indexData__pb2.JSONResponse.SerializeToString,
      ),
      'getSymbols': grpc.unary_unary_rpc_method_handler(
          servicer.getSymbols,
          request_deserializer=indexData__pb2.JSONRequest.FromString,
          response_serializer=indexData__pb2.Symbols.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'indexData.indexData', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# python 结构体定义
from enum import Enum
from dataclasses import dataclass

import _stock_load as lib

# 购或沽
class Type(Enum):
    # 股票
    Stock           = lib.Type.Stock
    # 指数
    Index           = lib.Type.Index
    # 基金
    Fund            = lib.Type.Fund
    # 期货
    Future          = lib.Type.Future
    # 期权
    Option          = lib.Type.Option
    # 债券
    Bond            = lib.Type.Bond
    # 可转换债券
    BondConvertible = lib.Type.BondConvertible

# 交易所
class Market(Enum):
    ShangHai        = lib.Market.ShangHai
    ShenZhen        = lib.Market.ShenZhen


# 股票信息
class Info:
    # 日期 yyyymmdd
    date:int
    # 时间 hhmmss
    time:int
    # 类型
    type:Type
    # 市场
    mk:Market
    # 合约代码(一般是数字, 也可能是字符, 如期货.)
    code:str
    # 开
    open:float
    # 高
    high:float
    # 低
    low:float
    # 收
    close:float
    # 数量
    volume:int 



# -*- coding: utf-8 -*-
# @Time    : 2017/1/24 16:55
# @Author  : hc
# @Site    :
# @File    : TestTrade.py
# @Software: PyCharm
# DLL是32位的,编译环境需要python 2.7以上的32位版本
# /// <param name="IP">券商交易服务器IP</param>
# /// <param name="Port">券商交易服务器端口</param>
# /// <param name="Version">设置通达信客户端的版本号</param>
# /// <param name="YybID">营业部代码，请到网址 http://www.chaoguwaigua.com/downloads/qszl.htm 查询</param>
# /// <param name="AccountNo">完整的登录账号，券商一般使用资金帐户或客户号</param>
# /// <param name="TradeAccount">交易账号，一般与登录帐号相同. 请登录券商通达信软件，查询股东列表，股东列表内的资金帐号就是交易帐号</param>
# /// <param name="JyPassword">交易密码</param>
# /// <param name="TxPassword">通讯密码</param>
# import TradeX
# import _TradeX
# client_id, err = TradeX.Logon("119.145.12.67", 443, "6", 11, "06118253378", "06118253378", "188188", "")
# client_id, err = TradeX.Logon("mock.tdx.com.cn", 7708, "6.40", 9000, "net828@163.com", "f001001001005792", "123123", "")
# if client_id < 0 :
#   print "fail"
#   err= err.decode('gb2312')
#   err =err.encode('utf-8')
#   print err
# else :
#   print "ok\n"
# ret, res, err = TradeX.TdxHq_Connect("14.17.75.71", 7709)
# print res.decode('gb2312').encode('utf-8')
# print err.decode('gb2312').encode('utf-8')
# #
# ret, res, err = TradeX.TdxHq_GetMinuteTimeData(0, "000001")
# print ret
# print res.decode('gb2312').encode('utf-8')
# print err.decode('gb2312').encode('utf-8')
# # print dir(TradeX)
# # print dir(_TradeX)
import ctypes
dll = ctypes.windll.LoadLibrary( 'trade20171030.dll' )
# dll =ctypes.cdll('trade20171030.dll')
IP="119.145.12.67"          #券商交易服务器IP
Port=443                    #券商交易服务器端口
Version="2.37"                 #设置通达信客户端的版本号
YybID=0                     #营业部代码
AccountNo="0618253378"     #完整的登录账号，券商一般使用资金帐户或客户号
TradeAccount="0618253378"  #交易账号
JyPassword="188188"          #交易密码
TxPassword=""           #通讯密码
ErrInfo=ctypes.create_string_buffer('/0' * 256)
# CBIQARGDEQ
#此API执行返回后，如果出错，保存了错误信息说明。一般要分配256字节的空间。没出错时为空字符串。
# ClientID =  dll.Logon("mock.tdx.com.cn", 7708, "6.40", 9000, "net828@163.com", "f001001001005792", "123123", "", ErrInfo)
# print dll.Logon(IP, Port,Version, YybID, AccountNo,TradeAccount, JyPassword, TxPassword, ErrInfo)
dll.OpenTdx()
# ErrInfo = ctypes.create_string_buffer('/0' * 256)
ClientID = dll.Logon(IP, Port,Version, YybID, AccountNo,TradeAccount, JyPassword, TxPassword, ErrInfo)
print(ClientID)
# print ErrInfo
print(ErrInfo.value.decode("gbk"))
# print(ClientID)
Result=ctypes.create_string_buffer('/0' * 1024)
dll.QueryData(ClientID, 4, Result, ErrInfo)# 查询资金
# ClientID.GetQuote()
# print (Result.value.decode("gbk"))
dll.QueryHistoryData(ClientID, 1, "20171025","20171030",Result,ErrInfo)# 查询各种历史数据
print(Result.value.decode("gbk"))
# <param name="ClientID">客户端ID</param>
# <param name="Category">表示委托的种类，0买入 1卖出  2融资买入  3融券卖出   4买券还券   5卖券还款  6现券还券</param>
# <param name="PriceType">表示报价方式 0上海限价委托 深圳限价委托 1(市价委托)深圳对方最优价格
#  2(市价委托)深圳本方最优价格  3(市价委托)深圳即时成交剩余撤销  4(市价委托)上海五档即成剩撤 深圳五档即成剩撤 5(市价委托)深圳全额成交或撤销 6(市价委托)上海五档即成转限价
# <param name="Gddm">股东代码, 交易上海股票填上海的股东代码；交易深圳的股票填入深圳的股东代码</param>
# <param name="Zqdm">证券代码</param>
# <param name="Price">委托价格</param>
# <param name="Quantity">委托数量</param
price  =ctypes.c_float()
price.value = 10.03
# dll.SendOrder(ClientID, 0, 0, "0121059858", "002864", price, 8500,Result,ErrInfo)
# import sys
# reload(sys)
# sys.getdefaultencoding('utf8')
# print (Result.value,ErrInfo.value.decode("gbk"))
# print(Result.value.decode("gbk").encode("utf8"))
# print(ErrInfo.value.decode("gbk").encode("utf8"))
# # print("\xe5\xa7\x94\xe6\x89\x98\xe4\xbb\xb7\xe6\xa0\xbc\xe9\x9d\x9e\xe6\xb3\x95".decode("gbk"))
# print("\xce\xaf\xcd\xd0\xb1\xe0\xba\xc5\t\xb7\xb5\xbb\xd8\xd0\xc5\xcf\xa2\t\xbc\xec\xb2\xe9\xb7\xe7\xcf\xd5\xb1\xea\xd6\xbe\t\xb1\xa3\xc1\xf4\xd0\xc5\xcf\xa2\n14\t\t\t".decode("gbk"))
# # 委托价格非法
# print "\xce\xaf\xcd\xd0\xbc\xdb\xb8\xf1\xb7\xc7\xb7\xa8".decode("gbk").encode("utf8")
# # 不存在的客户号或者客户号和股东代码不匹配!
# print "\xb2\xbb\xb4\xe6\xd4\xda\xb5\xc4\xbf\xcd\xbb\xa7\xba\xc5\xbb\xf2\xd5\xdf\xbf\xcd\xbb\xa7\xba\xc5\xba\xcd\xb9\xc9\xb6\xab\xb4\xfa\xc2\xeb\xb2\xbb\xc6\xa5\xc5\xe4!".decode("gbk")
print(dll.CloseTdx())



# -*- coding: utf-8 -*-
import os
import os.path
import shutil
import struct

# 遍历路径
def WalkInDir(nowDir, func):
    i = 0
    for parent,dirnames,filenames in os.walk(nowDir):
        for _dirname in dirnames:
            pass
        for filename in filenames:
            i += 1
            fullFileName = os.path.join(parent, filename)
            func(i, parent, filename, fullFileName)

# 遍历路径包含文件夹,负数为文件夹,正数为文件
def WalkInDirIncludeAll(nowDir, func):
    i = 0
    j = 0
    if os.path.exists(nowDir):
        # 当前路径也算
        j -= 1
        func(j, '', nowDir, nowDir)

    for parent,dirnames,filenames in os.walk(nowDir):
        for dirname in dirnames:
            j -= 1
            fullFileName = os.path.join(parent, dirname)
            func(j, parent, dirname, fullFileName)
        for filename in filenames:
            i += 1
            fullFileName = os.path.join(parent, filename)
            func(i, parent, filename, fullFileName)

# 判断文件是否存在
def IsHaveFile(fileWithPath):
    return os.path.isfile(fileWithPath)

# 判断路径是否存在
def IsHaveDir(path):
    return os.path.exists(path)

# 创建路径
def MakeDirs(path):
    if not IsHaveDir(path):
        os.makedirs(path)

# 删除指定路径下面的所有文件和目录
def DeleteAllFilesAndDirs(path):
    if IsHaveDir(path):
        shutil.rmtree(path)

# 删除指定的文件
def DeleteFile(fileWithPath):
    if IsHaveFile(fileWithPath):
        os.remove(fileWithPath)
        
# 拷贝文件到另一个目录下
def CopyFileToDst(fileSrc, dirDst):
    if not IsHaveFile(fileSrc):
        print('[error] copy file['+str(fileSrc)+'] not exist')
        return False
    MakeDirs(dirDst)
    fileName= os.path.split(fileSrc)[1]
    fileDst = os.path.join(dirDst, fileName)
    fr = open(fileSrc, 'rb')
    frInfo = fr.read()
    fw = open(fileDst, 'wb')
    fw.write(frInfo)
    fw.close()
    fr.close()
    return True

# 获取文件内容
def ReadFileData(filePath):
    if not IsHaveFile(filePath):
        print('[错误] 无法读取文件内容 [' + filePath + ']')
        return False
    f = open(filePath, 'rb')
    readInfo = f.read()
    f.close()
    return readInfo

# 获取文件字符内容
def ReadFileStr(filePath):
    if not IsHaveFile(filePath):
        print('[错误] 无法读取文件内容 [' + filePath + ']')
        return False
    f = open(filePath, 'r')
    readInfo = f.read()
    f.close()
    return readInfo

#
def ReadFileDataLines(filePath):
    if not IsHaveFile(filePath):
        print('[err] not found file [' + filePath + ']')
        return []
    try:
        f = open(filePath, 'rb')
        lines = f.readlines()
        f.close()
    except OSError:
        # 读取错误
        return []

    return lines

# 获取文件字符行内容
def ReadFileStrLines(filePath):
    if not IsHaveFile(filePath):
        print('[err] not found file [' + filePath + ']')
        return []
    try:
        f = open(filePath, 'rb')
        lines = f.readlines()
        f.close()
    except UnicodeDecodeError:
        # 二进制文件
        return []
    except OSError:
        # 读取错误
        return []

    return lines

#! 注意: 并未检查路径是否存在
def WriteFileInfo(filePath, fileInfo):
    #! 并未检查路径是否存在
    f = open(filePath, 'wb')
    f.write(fileInfo)
    f.close()
    
#! 注意: 并未检查路径是否存在
def WriteFileStrInfo(filePath, fileInfo):
    #! 并未检查路径是否存在
    f = open(filePath, 'w')
    f.write(fileInfo)
    f.close()    

# 读取二进制整数(网络字节序)
def ReadInt(binData):
    result = struct.unpack('!i', binData)
    print(str(result))
    return result[0]

# 生成二进制整数(网络字节序)
def MakeInt(iVal):
    return struct.pack('!i', iVal)

# 生成二进制浮点数(网络字节序)
def MakeFloatNet(r):
    return struct.pack('!f', r)

# 生成二进制浮点数(本地字节序)
def MakeFloatLocal(r):
    return struct.pack('=f', r)

# 获取文件扩展名
def GetFileExtname(fileName):
    extName = os.path.splitext(fileName)[-1]
    return extName
    



# -*-coding:utf-8-*-
import stomp
import time


queue_name = '/queue/SampleQueue'
topic_name = '/topic/SampleTopic'
listener_name = 'SampleListener'

class SampleListener(object):
    def on_message(self, headers, message):
        print ('headers: %s' % headers)
        print ('message: %s' % message)

# 推送到队列queue
def send_to_queue(msg):
    conn = stomp.Connection10([('127.0.0.1',61613)])
    conn.start()
    conn.connect()
    conn.send(queue_name, msg)
    conn.disconnect()

#推送到主题
def send_to_topic(msg):
    conn = stomp.Connection10([('127.0.0.1',61613)])
    conn.start()
    conn.connect()
    conn.send(topic_name, msg)
    conn.disconnect()

##从队列接收消息
def receive_from_queue():
    conn = stomp.Connection10([('127.0.0.1',61613)])
    conn.set_listener(listener_name, SampleListener())
    conn.start()
    conn.connect()
    conn.subscribe(queue_name)
    time.sleep(1) # secs
    conn.disconnect()

##从主题接收消息
def receive_from_topic():
    conn = stomp.Connection10([('127.0.0.1',61613)])
    conn.set_listener(listener_name, SampleListener())
    conn.start()
    conn.connect()
    conn.subscribe(topic_name)
    while 1:
        send_to_topic('topic')
        time.sleep(3) # secs

    conn.disconnect()

if __name__=='__main__':
    # send_to_queue('len 123')
    # receive_from_queue()

    receive_from_topic()


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import api.tradeApi_pb2 as tradeApi__pb2


class TradeStub(object):
    """The greeting service definition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.sayHello = channel.unary_unary(
                '/Trade.Trade/sayHello',
                request_serializer=tradeApi__pb2.JSONRequest.SerializeToString,
                response_deserializer=tradeApi__pb2.JSONResponse.FromString,
                )
        self.getPos = channel.unary_unary(
                '/Trade.Trade/getPos',
                request_serializer=tradeApi__pb2.JSONRequest.SerializeToString,
                response_deserializer=tradeApi__pb2.RspPosition.FromString,
                )
        self.getOrders = channel.unary_unary(
                '/Trade.Trade/getOrders',
                request_serializer=tradeApi__pb2.ReqQuery.SerializeToString,
                response_deserializer=tradeApi__pb2.RspOrders.FromString,
                )
        self.getTrans = channel.unary_unary(
                '/Trade.Trade/getTrans',
                request_serializer=tradeApi__pb2.ReqQuery.SerializeToString,
                response_deserializer=tradeApi__pb2.RspTrans.FromString,
                )
        self.getOrder_stream = channel.unary_stream(
                '/Trade.Trade/getOrder_stream',
                request_serializer=tradeApi__pb2.JSONRequest.SerializeToString,
                response_deserializer=tradeApi__pb2.RspOrders.FromString,
                )
        self.getTrans_stream = channel.unary_stream(
                '/Trade.Trade/getTrans_stream',
                request_serializer=tradeApi__pb2.JSONRequest.SerializeToString,
                response_deserializer=tradeApi__pb2.RspTrans.FromString,
                )
        self.cancelOrder = channel.unary_unary(
                '/Trade.Trade/cancelOrder',
                request_serializer=tradeApi__pb2.ReqCancelOrder.SerializeToString,
                response_deserializer=tradeApi__pb2.RspCancelOrder.FromString,
                )
        self.sendOrder = channel.unary_unary(
                '/Trade.Trade/sendOrder',
                request_serializer=tradeApi__pb2.ReqSentOrder.SerializeToString,
                response_deserializer=tradeApi__pb2.RspSentOrder.FromString,
                )


class TradeServicer(object):
    """The greeting service definition.
    """

    def sayHello(self, request, context):
        """Sends a greeting
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getPos(self, request, context):
        """获取仓位情况 - proto
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getOrders(self, request, context):
        """获取报单情况 - proto
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getTrans(self, request, context):
        """获取成交情况 - proto
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getOrder_stream(self, request, context):
        """服务端推送报单数据 - stream
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getTrans_stream(self, request, context):
        """服务端推送交易数据 - stream
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def cancelOrder(self, request, context):
        """撤单请求
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def sendOrder(self, request, context):
        """报单请求
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TradeServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'sayHello': grpc.unary_unary_rpc_method_handler(
                    servicer.sayHello,
                    request_deserializer=tradeApi__pb2.JSONRequest.FromString,
                    response_serializer=tradeApi__pb2.JSONResponse.SerializeToString,
            ),
            'getPos': grpc.unary_unary_rpc_method_handler(
                    servicer.getPos,
                    request_deserializer=tradeApi__pb2.JSONRequest.FromString,
                    response_serializer=tradeApi__pb2.RspPosition.SerializeToString,
            ),
            'getOrders': grpc.unary_unary_rpc_method_handler(
                    servicer.getOrders,
                    request_deserializer=tradeApi__pb2.ReqQuery.FromString,
                    response_serializer=tradeApi__pb2.RspOrders.SerializeToString,
            ),
            'getTrans': grpc.unary_unary_rpc_method_handler(
                    servicer.getTrans,
                    request_deserializer=tradeApi__pb2.ReqQuery.FromString,
                    response_serializer=tradeApi__pb2.RspTrans.SerializeToString,
            ),
            'getOrder_stream': grpc.unary_stream_rpc_method_handler(
                    servicer.getOrder_stream,
                    request_deserializer=tradeApi__pb2.JSONRequest.FromString,
                    response_serializer=tradeApi__pb2.RspOrders.SerializeToString,
            ),
            'getTrans_stream': grpc.unary_stream_rpc_method_handler(
                    servicer.getTrans_stream,
                    request_deserializer=tradeApi__pb2.JSONRequest.FromString,
                    response_serializer=tradeApi__pb2.RspTrans.SerializeToString,
            ),
            'cancelOrder': grpc.unary_unary_rpc_method_handler(
                    servicer.cancelOrder,
                    request_deserializer=tradeApi__pb2.ReqCancelOrder.FromString,
                    response_serializer=tradeApi__pb2.RspCancelOrder.SerializeToString,
            ),
            'sendOrder': grpc.unary_unary_rpc_method_handler(
                    servicer.sendOrder,
                    request_deserializer=tradeApi__pb2.ReqSentOrder.FromString,
                    response_serializer=tradeApi__pb2.RspSentOrder.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'Trade.Trade', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Trade(object):
    """The greeting service definition.
    """

    @staticmethod
    def sayHello(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Trade.Trade/sayHello',
            tradeApi__pb2.JSONRequest.SerializeToString,
            tradeApi__pb2.JSONResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getPos(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Trade.Trade/getPos',
            tradeApi__pb2.JSONRequest.SerializeToString,
            tradeApi__pb2.RspPosition.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getOrders(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Trade.Trade/getOrders',
            tradeApi__pb2.ReqQuery.SerializeToString,
            tradeApi__pb2.RspOrders.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getTrans(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Trade.Trade/getTrans',
            tradeApi__pb2.ReqQuery.SerializeToString,
            tradeApi__pb2.RspTrans.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getOrder_stream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/Trade.Trade/getOrder_stream',
            tradeApi__pb2.JSONRequest.SerializeToString,
            tradeApi__pb2.RspOrders.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getTrans_stream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/Trade.Trade/getTrans_stream',
            tradeApi__pb2.JSONRequest.SerializeToString,
            tradeApi__pb2.RspTrans.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def cancelOrder(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Trade.Trade/cancelOrder',
            tradeApi__pb2.ReqCancelOrder.SerializeToString,
            tradeApi__pb2.RspCancelOrder.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def sendOrder(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/Trade.Trade/sendOrder',
            tradeApi__pb2.ReqSentOrder.SerializeToString,
            tradeApi__pb2.RspSentOrder.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# 交易周期
from enum import Enum
class SUBSTITUTE_FLAG(Enum): # 现金替代标志
    ALLOW = 0           # 允许
    MUST = 1           # 必须
    FORBID = 2           # 禁止

class STK_STATUS(Enum):  # 股票状态
    NORMAL = 0           # 正常
    LIM_UP = 1           # 涨停
    LIM_DOWN = 2         # 跌停
    SUSPEND = 3          # 停牌



class PERIOD(Enum):
    s1 = "second1"
    m1 = "minite1"
    d1 = "day1"

class TRADE_TYPE(Enum):
    open  = 0
    close = 1

class CALL_OR_PUT(Enum):
    nil = 'NaN'
    call= 'call'
    put = 'put'

class DIR(Enum):
    buy     = 0
    sell    = 1
    both    = 2
    null    = 3

class MARKET(Enum):
    stock   = "stock"
    future  = "future"
    option  = "option"

class MODE(Enum):
    public      = "public"
    backtesting = "backtesting"

class ORDER_TYPE(Enum):
    LIMIT  = 0 # 限价委托
    MARKET = 1 # 市价委托

class ORDER_STATUS(Enum):
    """报单状态"""
    PENDING     = 0 # 待成交
    TRADED      = 1 # 已经全部成交
    TRADED_PART = 2 # 部分成交
    CANCEL_REQ  = 3 # 待撤单
    CANCELED    = 4 # 已撤单
    REJECTED    = 5 # 被拒绝
    FAILURE     = 6 # 已失败（网络问题等技术问题导致的失败）

class DATA_TYPE(Enum):
    """数据类型"""
    hq      = "hq"      # 行情
    hhq     = "hhq"     # 高速行情
    order   = "order"   # 报单
    tran    = "tran"    # 成交
    index   = "index"   # 指数
    info    = "info"    # 股票信息
    div     = "div"     # 股票分红送配
    k_min   = "k_min"    # K线
    k_day   = "k_day"    # K线

# 返回值状态类型
class RET_TYPE(Enum):
    """数据类型"""
    NORMAL       = 0   # 正常
    ERR_NORMAL   = 1   # 一般错误
    ERR_SERIOUS  = 2   # 严重错误
    ERR_IGNORE   = 3   # 忽略错误 - 做心跳.

MAX_POSITION_NUMBER = 9999999999
USE_REFERENDUM_PRICE = False
MAX_STOCK_CODE = 1700000

# USE_DATACENTER 控制datacenter 读取数据的模式
USE_DATACENTER = "mysql"
# USE_DATACENTER = "redis"

LOG_ERROR = "error"
LOG_DATA = "data_error"
LOG_SQL = "sql"
LOG_TRADE = "trade"
LOG_TRANSECTION = "transection"
LOG_TRADE_ERROR = 'trade_error'
LOG_ORDER = "order"
LOG_GATEWAY = "gateway"
LOG_RECORD = "trade_record"
LOG_POLICY = "trade_policy"
LOG_POSITION = "position"
LOG_POSITION_CHANGE = "position_change"
LOG_DAY_NET_VALUE = "day_net_value"
LOG_MONTH_NET_VALUE = "month_net_value"
LOG_YEAR_NET_VALUE = "year_net_value"
LOG_SUMMARY_REPORT = "summary_report"
LOG_EACH_STOCK_PROFIT = "each_stock_profit"
LOG_CLOSE_PROFIT = "close_profit"

LOG_DIVIEND_DIVIEND = "diviend_diviend"
LOG_DIVIEND_TRANSFER = "diviend_transfer"
LOG_DIVIEND_QTY_COST = "diviend_qty_cost"
LOG_DIVIEND_BUY = "diviend_buy"
LOG_DIVIEND_BUY_FAILED = "diviend_buy_failed"
LOG_DATACENTER = "dataCenter"
LOG_MQRECEIVER = "mqReceiver"

g_log_switch = {
    LOG_ERROR : True,
    LOG_DATA : False,
    LOG_TRADE : True,
    LOG_RECORD : True,
    LOG_POSITION_CHANGE : True,
    LOG_EACH_STOCK_PROFIT : True,
    LOG_DIVIEND_DIVIEND : True,
    LOG_DIVIEND_TRANSFER : True,
    LOG_DIVIEND_QTY_COST : False,
    LOG_DIVIEND_BUY : True,
    LOG_DIVIEND_BUY_FAILED : False,
}


g_report_switch = {
    LOG_RECORD : False,
    LOG_MONTH_NET_VALUE : True,
    LOG_YEAR_NET_VALUE : True,
    LOG_SUMMARY_REPORT: True,
    LOG_DAY_NET_VALUE: True,
}

def is_log_switch_on(log_name):
    return g_report_switch.get(log_name, False)

def enum(**enums):
    return type('Enum', (), enums)

EVENT_TICK          = 'eTick.'          # TICK行情事件，可后接具体的vtSymbol
EVENT_TRADE         = 'eTrade.'         # 成交回报事件
EVENT_ORDER         = 'eOrder.'         # 报单回报事件
EVENT_ORDER_STATUS  = 'eOrderStatus'    # 报单状态
EVENT_POSITION      = 'ePosition.'      # 持仓回报事件
EVENT_TRANSACTION   = 'eTransaction'    # 成交回报事件
EVENT_ACCOUNT       = 'eAccount.'       # 账户回报事件
EVENT_CONTRACT      = 'eContract.'      # 合约基础信息回报事件
EVENT_ERROR         = 'eError.'         # 错误回报事件
EVENT_COMMAND       = 'eCommand'        # 交易
EVENT_TIMER         = 'eTimer'          # 时间心跳事件
EVENT_DATA          = 'eData.'          # 数据接收事件

if __name__ == '__main__':
    a = enum(**g_report_switch)
    print (dir(a))
    print(STK_STATUS(1))


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tradex.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tradex.proto',
  package='TradeX',
  syntax='proto3',
  serialized_options=_b('\n\027io.grpc.examples.tradexB\013TradeXProtoP\001\242\002\003TDX'),
  serialized_pb=_b('\n\x0ctradex.proto\x12\x06TradeX\"!\n\x0bJSONRequest\x12\x12\n\nrqt_string\x18\x01 \x01(\t\"\"\n\x0cJSONResponse\x12\x12\n\nrsp_string\x18\x01 \x01(\t2\xec\x01\n\x06TradeX\x12\x37\n\x08SayHello\x12\x13.TradeX.JSONRequest\x1a\x14.TradeX.JSONResponse\"\x00\x12\x35\n\x06GetPos\x12\x13.TradeX.JSONRequest\x1a\x14.TradeX.JSONResponse\"\x00\x12\x38\n\tGetOrders\x12\x13.TradeX.JSONRequest\x1a\x14.TradeX.JSONResponse\"\x00\x12\x38\n\tSendOrder\x12\x13.TradeX.JSONRequest\x1a\x14.TradeX.JSONResponse\"\x00\x42.\n\x17io.grpc.examples.tradexB\x0bTradeXProtoP\x01\xa2\x02\x03TDXb\x06proto3')
)




_JSONREQUEST = _descriptor.Descriptor(
  name='JSONRequest',
  full_name='TradeX.JSONRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rqt_string', full_name='TradeX.JSONRequest.rqt_string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=57,
)


_JSONRESPONSE = _descriptor.Descriptor(
  name='JSONResponse',
  full_name='TradeX.JSONResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='rsp_string', full_name='TradeX.JSONResponse.rsp_string', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=59,
  serialized_end=93,
)

DESCRIPTOR.message_types_by_name['JSONRequest'] = _JSONREQUEST
DESCRIPTOR.message_types_by_name['JSONResponse'] = _JSONRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

JSONRequest = _reflection.GeneratedProtocolMessageType('JSONRequest', (_message.Message,), dict(
  DESCRIPTOR = _JSONREQUEST,
  __module__ = 'tradex_pb2'
  # @@protoc_insertion_point(class_scope:TradeX.JSONRequest)
  ))
_sym_db.RegisterMessage(JSONRequest)

JSONResponse = _reflection.GeneratedProtocolMessageType('JSONResponse', (_message.Message,), dict(
  DESCRIPTOR = _JSONRESPONSE,
  __module__ = 'tradex_pb2'
  # @@protoc_insertion_point(class_scope:TradeX.JSONResponse)
  ))
_sym_db.RegisterMessage(JSONResponse)


DESCRIPTOR._options = None

_TRADEX = _descriptor.ServiceDescriptor(
  name='TradeX',
  full_name='TradeX.TradeX',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=96,
  serialized_end=332,
  methods=[
  _descriptor.MethodDescriptor(
    name='SayHello',
    full_name='TradeX.TradeX.SayHello',
    index=0,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_JSONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetPos',
    full_name='TradeX.TradeX.GetPos',
    index=1,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_JSONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetOrders',
    full_name='TradeX.TradeX.GetOrders',
    index=2,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_JSONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SendOrder',
    full_name='TradeX.TradeX.SendOrder',
    index=3,
    containing_service=None,
    input_type=_JSONREQUEST,
    output_type=_JSONRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TRADEX)

DESCRIPTOR.services_by_name['TradeX'] = _TRADEX

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# @Time    : 2018/11/15 14:41
# @Author  : hc
# @Site    : 
# @File    : StockInfo.py
# @Software: PyCharm

import requests
from bs4 import BeautifulSoup

STOCK_URL= "http://quote.eastmoney.com/stocklist.html" #股票代码查询一览表

class StockInfo():
    def __init__(self):
        self.url =STOCK_URL
    def get_data(self):
        response = requests.get(self.url,)


        self.format(response.content)

    def format(self, rep_data):
        soup = BeautifulSoup(rep_data, "lxml")
        quote_body = soup.select('quote_body')[0]
        print(quote_body)




if __name__ == '__main__':
    stock = StockInfo()
    stock.get_data()


# -*- coding: utf-8 -*-
import os,types,sys,time,stat
import shutil as sh

"""
    函数：try_copytree(src,dst)
    功能：递归拷贝文件夹到另一个文件夹下，如果存在相同的文件夹/文件，则全部覆盖!本函数修改自
    shutil.copytree 原始函数因为不能拷贝已经存在的目录因此不适用本系统
    如果源地址是文件，则目标地址也必须是文件
    如果源地址是目录，则目标地址也必须是目录
    其中的路径格式必须使用 unix 风格路径，已 ‘/’ 分割
"""
def try_copytree(src, dst):
    if os.path.isfile(src):
        dstSeg   = os.path.split(dst)
        dstPath  = ''.join(dstSeg[:-1])
        fileName = dstSeg[-1:]
        if not os.path.exists(dstPath):
            os.makedirs(dstPath)
        sh.copy2(src, dst)
    elif os.path.exists(src):
        names = os.listdir(src)
        if not os.path.isdir(dst):
            os.makedirs(dst)
        for name in names:
            srcname = os.path.join(src, name)
            dstname = os.path.join(dst, name)
            if os.path.isdir(srcname):
                try_copytree(srcname, dstname)
            else:
                sh.copy2(srcname, dstname)
# 
def cj_copytree(src, dst):
    #print '原文拷贝 '+src+' '+dst
    try_copytree(src,dst)

# 
def try_rename(src, dst):
    try:
        head, tail = os.path.split(dst)
        if head and tail and not os.path.exists(head):
            os.makedirs(head)
        os.rename(src, dst)
        #print '直接重命名 '+src+' '+dst
    except OSError:
        cj_copytree(src,dst)
        
"""
    函数：print_s()
    功能：格式化 PY 的 结构，成一个标准字符串
    参数：obj    -> 一个正规的 Py 结构（字典，列表，元组）
         level  -> 层级缩近的初始值 默认为 0
         s      -> 当前已经序列化完毕的字符串 默认为 空字串
"""
def print_s(obj,level,s):
    dot_tag   = ','         #   定义元素分割符
    ret_tag   = '\n'        #   定义换行符
    quo_tag   = '\''        #   定义字符串引用符号
    cut_tag   = ':'         #   定义键值分割符
    u_s_tuple = ('(',')')   #   定义元组起始结束符
    u_s_list  = ('[',']')   #   定义列表起始结束符
    u_s_dict  = ('{','}')   #   定义字典起始结束符
    fan_all   = u_s_tuple[1]+u_s_list[1]+u_s_dict[1]
    space = ''
    for i in range(0,level):
        space += '  '
    level += 1
    tp = type(obj)
    o_len = len(obj)-1
    if tp == list or tp == set:
        for k in range(len(obj)):
            tp2 = type(obj[k])
            if tp2 == list:
                s.append(ret_tag+space+u_s_list[0])
                s  = [print_s(obj[k],level,s)]
                if s[len(s)-1][-1:] in fan_all:
                    s.append(ret_tag+space)
                s.append(u_s_list[1])
            elif tp2 == set:
                s.append(ret_tag+space+u_s_tuple[0])
                s  = [print_s(obj[k],level,s)]
                if s[len(s)-1][-1:] in fan_all:
                    s.append(ret_tag+space)
                s.append(u_s_tuple[1])
            elif tp2 == dict:
                s.append(u_s_dict[0])
                s  = [print_s(obj[k],level,s)+ret_tag+space+u_s_dict[1]]
            elif tp2 == str:
                s.append(quo_tag+str(obj[k])+quo_tag)
            else:
                s.append(str(obj[k]))
            if k < o_len:
                s.append(dot_tag)
    elif tp == dict:
        ii = 0
        for key in obj:
            tp2 = type(obj[key])
            if type(key) == str:
                keys = quo_tag+key+quo_tag
            else:
                keys = str(key)
            if tp2 == list:
                s.append(ret_tag+space+keys+cut_tag+u_s_list[0])
                s  = [print_s(obj[key],level,s)]
                if s[len(s)-1][-1:] in fan_all:
                    s.append(ret_tag+space)
                s.append(u_s_list[1])
            elif tp2 == set:
                s.append(ret_tag+space+keys+cut_tag+u_s_tuple[0])
                s  = [print_s(obj[key],level,s)]
                if s[len(s)-1][-1:] in fan_all:
                    s.append(ret_tag+space)
                s.append(u_s_tuple[1])
            elif tp2 == dict:
                s.append(ret_tag+space+keys+cut_tag+u_s_dict[0])
                s  = [print_s(obj[key],level,s)+ret_tag+space+u_s_dict[1]]
            elif tp2 == str:
                s.append(ret_tag+space+keys+cut_tag+quo_tag+str(obj[key])+quo_tag)
            else:
                s.append(ret_tag+space+keys+cut_tag+str(obj[key]))
            if ii < o_len:
                s.append(dot_tag)
            ii += 1
    elif tp == str:
        s.append(space+quo_tag+obj+quo_tag)
    else:
        s.append(space+str(obj))
    return ''.join(s)

"""
    格式化输出PYTHON 的结构对象，可以处理字典，列表，元组等类型是仿照PHP 的 print_r 所写的
    ：）
"""
def print_r(obj):
    if type(obj) == dict:
        print('{'+print_s(obj,1,[])+'\n}')
    elif type(obj) == list:
        print('['+print_s(obj,1,[])+']')
    elif type(obj) == set:
        print('('+print_s(obj,1,[])+')')
    else:
        print(str(obj))

class Log:
    """
        日志记录类： Log
        功能： 向指定文件写入 LOG 记录用来用来记载程序执行流程的细节是否顺利
        自动分组，自定义缓存大小，自定义是否输出
    """
    ch_num = 100        #   定时清理缓存
    change = 5000       #   定义文件分组大小
    def __init__(self,f):
        self.f = f      #   记录 LOG 所使用的文件名
        dirs,files = os.path.split(f)
        if dirs and not os.path.isdir(dirs):
            os.makedirs(dirs)
        file(self.f,'w').close()
        self.n = 0      #   记录 LOG 的次数
        self.g = 1      #   记录当前分组次数
        self.c = []     #   记录 LOG 使用的缓存
        self.l = len(f) #   取得当前初始文件名长度
        self.p_map = {
            'yes':self._p,
            'no' :self._f
        }
    def write(self,log,if_print='yes'):
        logt = '\n'+(str(time.localtime()).replace(',','-',2).replace(',',' ',1).replace(',',':',2)[1:-12])+'\n\t'+log
        self.p_map[if_print](log)
        self.c.append(logt)
        self.n += 1
        if self.n % self.ch_num == 0:
            self._r()
    def finish(self):
        self._r()
    def _r(self):
        if self.n / self.change > self.g:
            self.f = self.f[:self.l]+'.'+str(self.n/self.change)+'.txt'
            self.g +=1
        fp = file(self.f,'a+')
        fp.write(''.join(self.c))
        fp.close()
        self.c = []
    def _f(self,log=''):
        pass
    def _p(self,log=''):
        print(log)
        
"""
    函数： error_break(msg)
    功能： 退出程序的执行并在屏幕打印用户字串
"""
def error_break(msg=''):
    l = Log('./error.txt')
    l.write(msg)
    l.finish()    
    anykey = raw_input('按任意键停止程序运行！:')
    sys.exit(0)
#   向后查找一个有效字符
def glt(fs,i):
    i -= 1
    r = fs[i]
    while i>=0 and (r == "\r" or r == "\n" or r == "\t" or r == ' '):
        i -= 1
        r = fs[i]
    return r
#   向前查找一个有效字符    
def gft(fs,i,l):
    i += 1
    r = fs[i]
    while i<l and (r == "\r" or r == "\n" or r == "\t" or r == ' '):
        i += 1
        r = fs[i]
    return r
#   查看当前所在字符是否没有被转义
def fa(ser,fr,ft):
    if ser[fr] != ft:
        return False
    r  = 0
    fr-= 1
    if fr<=0:
        return True
    while ser[fr] == '\\':
        if fr<=0:
            return True
        r += 1
        fr-= 1
    if r%2 == 0:
        return True
    else:
        return False
#   获取一个文件的扩展名
def get_en(file_name):
    fn = file_name.rfind('.')
    if fn >= -1:
        return file_name[fn:]
    else:
        return ''
# End of ms_inc.py



'''
    这里测试了从历史行情仓库获取历史数据的接口.

    测试:
        1. 进入python3.6 环境
            F:/ProgramFiles/Anaconda3/Scripts/activate
            conda activate F:\ProgramFiles\Anaconda3
        
        2. 测试命令
            python test_py_hq_client.py

'''
import time
from py_hq_client import *

print("begin work")

# 测试获取某日的分钟线行情信息
info = hq_client_fetch(BAR_TYPE.Minute, 
                       CodeInfo(MARKET_TYPE.ShangHai, HQ_TYPE.Stock, 510050), 
                       TimePos(20201201, 93000000), 
                       TimePos(20210115, 150000000))

# 测试订阅标的的实时行情信息
hq_client_subscribe(CodeInfo(MARKET_TYPE.ShangHai, HQ_TYPE.Stock, 510050))

# 获取当日标的的情况
ltime = time.localtime()
yyyymmdd = int(time.strftime('%Y%m%d', ltime))
optInfos = hq_client_get_day_opt_info(yyyymmdd)

if optInfos and len(optInfos):
    for optInfo in optInfos:
        # 订阅当日期权信息
        hq_client_subscribe(CodeInfo(MARKET_TYPE.ShangHai, HQ_TYPE.Option, optInfo.code))


while True:
    if not hq_client_is_connect():
        # 连接已经断开
        break
    
    hq = hq_client_try_take()
    if not hq:
        # 未获取到行情
        # 模拟推送一个行情
        # hq_client_push(BAR_TYPE.Tick, HQInfo(mk= MARKET_TYPE.ShangHai, 
        #     hq= HQ_TYPE.Stock, 
        #     code = 510050,
        #     date = 20200102, 
        #     time = 15,
        #     open = 1.0,
        #     high = 2.0,
        #     low = 3.0,
        #     close = 4.0,
        #     volume = 5.0))
        time.sleep(0.1)
        continue

    print(str(hq.cinfo.code) + ' ' + str(hq.close))


print("all ok end work")


# -*- coding: utf-8 -*-
# 简单实现
# https://zhuanlan.zhihu.com/p/60499205
# 原理
# https://mp.weixin.qq.com/s?__biz=MzIxMDM0Mjg1Mw==&mid=2247483730&idx=1&sn=d32547775c275fad11ad0b724535e25f&chksm=976747f8a010ceeed594e1ca98fdbeb580132cd2531753b108adf8e1f73ad4e95c3d5e737e8f&scene=178&cur_album_id=1524209730238382081#rd
# 贝叶斯优化
# https://zhuanlan.zhihu.com/p/38282835
# https://zhuanlan.zhihu.com/p/363540266
from os import stat
import pandas as pd
import numpy as np
from pandas.core.frame import DataFrame
from pandas.core.series import Series
from tradeTools import Decorator
from tradeTools import helpers
from tradeTools import indexData
from tradeTools import DataMgr
import scipy.optimize as sco
import matplotlib.pyplot as plt
from pypfopt import risk_models
from pypfopt import expected_returns
from pypfopt.efficient_frontier import EfficientFrontier


class MeanVariance():
    def __init__(self) -> None:
        pass
    
    def optWeights(self,df:DataFrame,coef:int = 252,type:str="avg",isSale=False,expectedRet =np.array([]))->DataFrame:
        """[利用马科维茨模型获取最优权重]

        Args:
            df (DataFrame): [收益率序列 ，每一列代表一只股票，每一行代表一天的收盘价]
            coef (int, optional): [转换为年化的系数。如果收益率序列式日频，默认值为252]. Defaults to 252.
            type (str, optional): [求解。平均权重/最小方差/最优夏普 "minVar","avg","minSharp"]. Defaults to "avg".
            isSale (bool, optional): [是否允许做空]
            expectedRet (list, optional): [期望的收益率]. Defaults to []. 如果为空。这期望收益率是历史收益率的均值
        Returns:
            DataFrame: [description]
        """
        if expectedRet.size != 0:
            rtn = expectedRet
        else:
            rtn = df.mean()
        def stats(weights):
            weights = np.array(weights)
            port_returns = np.sum(rtn*weights)*coef # 年化收益率
            port_variance = np.sqrt(np.dot(weights.T, np.dot(df.cov()*coef,weights))) # 年化标准差
            return np.array([port_returns, port_variance, port_returns/port_variance])

        #给定初始权重
        cnt = df.shape[1]
        x0 = cnt*[1./cnt] # 初始权重。平均分配
        if type!="avg":
            if type =="minVar":
                def minFunc(weights):
                    return stats(weights)[1]**2
            else : # type =="minSharpe"
                def minFunc(weights):
                    return -stats(weights)[2] #最小化夏普指数的负值
           
            if isSale:
                bnds = ()
            else:
                 #权重（某股票持仓比例）限制在0和1之间。
                bnds = tuple((0,0.2) for x in range(cnt))
            #权重（股票持仓比例）的总和为1。
            cons = ({'type':'eq', 'fun':lambda x: np.sum(x)-1})
            #优化函数调用中忽略的唯一输入是起始参数列表(对权重的初始猜测)。我们简单的使用平均分布。
            opts = sco.minimize(minFunc,
                                x0, 
                                method = 'SLSQP', 
                                bounds = bnds, 
                                constraints = cons)
            w = opts["x"]
        else:
            w = x0
        symbolWeights = pd.DataFrame({"code":df.columns,
        "weight":w.round(3)}
        )
        symbolWeights.set_index('code', inplace=True)
        print("权重：",w.round(3))
        print("预期收益率，标准差，夏普率：",stats(w).round(3))        
        return symbolWeights
    
def genSymbolsReturn(symbols):
    dfs = []
    m = DataMgr.Mgr() 
    for symbol  in symbols:
        # TODO是否要用log计算对数收益率
        rate = m.genSymbolRateData(symbol=symbol,frequency=1)
        rate = rate[("20160528"<=rate.index)&(rate.index<="20201226")]
        rate = rate["mRate"]
        rate.name = symbol
        dfs.append(pd.DataFrame(rate))
    symbolsReturn = pd.concat(dfs,axis=1) 
    # symbolsReturn = symbolsReturn.dropna()
    return symbolsReturn    

# 获取最后一日市值权重比例
def genMarketValueWeight(symbols):
    dfs = []
    m = DataMgr.Mgr() 
    for symbol  in symbols:
        # TODO是否要用log计算对数收益率
        rate = m.genSymbolRateData(symbol=symbol,frequency=1)
        rate = rate[("20180528"<=rate.index)&(rate.index<="20200326")]
        marketValue = rate["MarketV"]
        marketValue.name = symbol
        dfs.append(pd.DataFrame(marketValue))
    marketValue = pd.concat(dfs,axis=1) 
    marketValue["TOTAL"] = marketValue.sum(axis=1) 
    marketValue = (marketValue.T / marketValue.TOTAL).T # 求权重
    marketValue = marketValue.iloc[-1]
    marketValue = marketValue.drop("TOTAL")
    return marketValue   

class BlackLitterman:
    # 计算风险厌恶系数lambda、先验预期收益率implied_ret
    def get_implied_excess_equilibrium_return(self, stock_cc_ret, w_mkt):
        '''
        :param stock_cc_ret: 指定T部分的10只股票收益率数据（维度：T * 10）
        :param w_mkt: 当前的市场权重（维度：1*10）
        :return: 风险厌恶系数lambd、先验预期收益率：implied_ret
        '''

        # weekly risk-free cc return = ln(1+3.24%)/(365/7) = 0.0006132
        # rf = (1+3.24/100)/(365) # 无风险日收益率
        rf = 0 # 无风险日收益率
        # 根据股票收益率计算得到协方差矩阵：mkt_cov
        mkt_cov = np.array(stock_cc_ret.cov())

        # lambd: implied risk-aversion coefficient（风险厌恶系数）
        # 即计算夏普率
        lambd = ((np.dot(w_mkt, stock_cc_ret.mean())) - rf) / np.dot(np.dot(w_mkt, mkt_cov), w_mkt.T)
        # 计算先验预期收益率：implied_ret
        implied_ret = lambd * np.dot(mkt_cov, w_mkt)
        return implied_ret, lambd
    
    # 设定观点矩阵 P、相对收益率向量 Q（共4种Views可选）
    # 目前 只有view_type =3 可用
    def get_views_P_Q_matrix(self,stock_cc_ret):
        N = stock_cc_ret.shape[1]
        
        T_near = 30
        P = np.identity(N)
        stock_cc_ret_near = stock_cc_ret.iloc[-T_near:]
        stock_cc_ret_near = stock_cc_ret_near.fillna(0)  # 停牌股。当日的收益率为0
        print("stock_cc_ret_near.mean()")
        print(stock_cc_ret_near.mean())
        Q = np.array(stock_cc_ret_near.mean())
        return P,Q    
    """
        if(view_type == 0 or view_type == 1):
            # view_type = 0: 投资者无观点，使用当前市值权重（即均衡状态下的权重）作为作为投资组合的权重
            # view_type = 1: 为投资者分配任意观点，这里随机分配了3个观点
            '''
            # 观点1. 伯克希尔哈撒韦比埃克森美孚的预期收益高0.01%；
            # 观点2. 微软比摩根大通的预期收益高0.025%；
            # 观点3. 10%摩根+90%VISA的投资组合比10%沃尔玛+90%美国银行的投资组合预期收益高0.01%
            # '''
            # P = np.zeros([3, N])
            # P[0, 8] = 1
            # P[0, 9] = -1
            # P[1, 1] = 1
            # P[1, 3] = -1
            # P[2, 3] = 0.1
            # P[2, 4] = 0.9
            # P[2, 6] = -0.1
            # P[2, 7] = -0.9
            # Q = np.array([0.0001, 0.00025, 0.0001])
        elif(view_type == 2):
          
            P = np.zeros([1, N])
            P[0, 2] = 1
            P[0, 3] = -1
            Q = [0.017]
        elif(view_type == 3):
            # view_type = 3: 选用最近VIEW_T期的历史平均收益率作为预期收益率
            # T_near: 使用近期T_near期数据的历史平均收益率作为预期收益率
            T_near = 30
            P = np.identity(N)
            stock_cc_ret_near = stock_cc_ret.iloc[-T_near:]
            Q = np.array(stock_cc_ret_near.mean())
        else:
            print("There is no such kind of view type!")
        return P, Q
    """
  # Step8. 计算Omega矩阵
    def get_views_omega(self, mkt_cov, P):
        tau = 0.3   # 后验期望收益率协方差矩阵的放缩尺度，取值在0~1之间
        # K: 投资者观点的数量
        K = len(P)
        # 生成K维度的对角矩阵（对角线上全为1）
        omega = np.identity(K)
        for i in range(K):
            # 逐行选取P（Views矩阵，维度：K*N，此处N=10）
            P_i = P[i]
            omg_i = np.dot(np.dot(P_i, mkt_cov), P_i.T) * tau
            # 将得到的结果赋值到矩阵对角线元素
            omega[i][i] = omg_i
        return omega
    
    # Step9. 计算后验期望收益率mu_p
    def calc_posterior_combined_return(self, mkt_cov, implied_ret, P, Q, omega):
        # tau为缩放尺度
        tau = 0.3   # 后验期望收益率协方差矩阵的放缩尺度，取值在0~1之间
        # 后验期望收益率mu_p的计算公式
        _tau = np.linalg.inv(tau * mkt_cov)
        _omega = np.linalg.inv(omega)
        k = np.linalg.inv(_tau + np.dot(np.dot(P.T, _omega), P))
        posterior_ret = np.dot(k, np.dot(np.linalg.inv(tau * mkt_cov), implied_ret) +
                            np.dot(np.dot(P.T, np.linalg.inv(omega)), Q))
        return posterior_ret   
    
        # Step10. 计算由BL模型得到的新权重weight_bl
    def get_weight_bl(self, posterior_ret, mkt_cov, lambd):
        weight_bl = np.dot(np.linalg.inv(lambd * mkt_cov), posterior_ret)
        return weight_bl
    
    def getPosteriorCombinedReturn(self,df:DataFrame,marketValueWeight:Series):
        mkt_cov = np.array(df.cov())                              # 将T区间部分的股票收益率计算成协方差矩阵

        implied_ret, lambd = self.get_implied_excess_equilibrium_return(stock_cc_ret=df,w_mkt=marketValueWeight)
        # print(f"先验收益率{implied_ret}")
        P,Q = self.get_views_P_Q_matrix(stock_cc_ret = df)
        omega = self.get_views_omega(mkt_cov = mkt_cov , P = P) 
        # 后验收益率
        posterior_ret = self.calc_posterior_combined_return(
            implied_ret = implied_ret,
            mkt_cov = mkt_cov, 
            P=P,
            Q=Q, 
           omega=omega
            )
        # print(f"后验收益率{posterior_ret}")
        return posterior_ret
        # w = self.get_weight_bl(posterior_ret, mkt_cov, lambd)
        
        # symbolWeights = pd.DataFrame({"code":df.columns,
        # "weight":w.round(3)}
        # )
        # print("权重：",w.round(3))
        
        # return symbolWeights
class Kelly:
    def optWeights(self,df):
        mean_returns = df.mean()
        cov_matrix = df.cov()
        precision_matrix = pd.DataFrame(np.linalg.inv(cov_matrix), index=df.columns.tolist(), columns=df.columns.tolist())
        kelly_wt = precision_matrix.dot(mean_returns)
        print(kelly_wt)
        maxV = kelly_wt.max()
        print(maxV)
        minV = kelly_wt.min()
        def f(x):
            return 1.0/(1+np.exp(-x))
        # def f(x):
        #     x = (x - minV) / (maxV - minV)
        #     return x
        kelly_wt = kelly_wt.apply(f)
        kelly_wt = kelly_wt/kelly_wt.sum()
        df = pd.DataFrame(kelly_wt,columns=["weight"])
        print(df)
        return df
if __name__ == '__main__':
    #===========测试用的收益率序列==========
    # symbols = [
    # 'SZ.000651',       ##格力电器   
    # 'SH.600519',       ##贵州茅台  
    # 'SH.601318',       ##中国平安               
    # 'SZ.000858',       ##五粮液  
    # 'SH.600887',       ##伊利股份  
    # 'SZ.000333',       ##美的集团   
    # 'SH.601166',       ##兴业银行  
    # 'SH.601328',       ##交通银行  
    # 'SH.600104'        ##上汽集团 
    # ]
    # df = genSymbolsReturn(
    #    symbols
    #     ) # 生成每日收益率序列
 
    # meanVariance = MeanVariance()
    # # 最优夏普
    # w = meanVariance.optWeights(df = df,type="minSharpe")
    # print("====MV=最优夏普====")
    # print(w)
    # # 最小方差
    # w = meanVariance.optWeights(df = df,type="minVar")
    # print("====MV=最小方差====")
    # print(w)
    # print("------------BlackLitterman------------")
    # print("------------利用贝叶斯收缩计算后验收益率然后代入马科维茨模型 ------------")
    # bl = BlackLitterman()
    # mk = genMarketValueWeight(symbols=symbols) # 最后一日数据的市值权重
    # # 计算后验夏普率
    # posteriorCombinedReturn = bl.getPosteriorCombinedReturn(df=df,marketValueWeight=mk)
    # print(posteriorCombinedReturn)
    # w = meanVariance.optWeights(df = df,type="minSharpe",expectedRet =posteriorCombinedReturn )
    # print(w)
    # w = meanVariance.optWeights(df = df,type="minVar",expectedRet =posteriorCombinedReturn )
    # print(w)
    # # ===========三方库===========
    # # # Optimize for maximal Sharpe ratio
    # mu = expected_returns.mean_historical_return(df,returns_data = True)
    # S = risk_models.sample_cov(df)
    # print(mu)
    # print(S)
    # ef = EfficientFrontier(mu, S)
    # raw_weights = ef.max_sharpe()
    # print(raw_weights)
    # k = Kelly()
    # k.optWeights(df = df)
    
# https://zhuanlan.zhihu.com/p/135320360
    p =0.6 # 胜率
    b = 1  # 赔率
    f =(p*b-(1-p))/b # 仓位值
    import math
    print("凯利值即仓位：",f)
    # 用 f代入
    maxf =  p*math.log(1+b*f)+(1-p)*math.log(1-f)
    # 用 0.5f 代入
    halfmaxf =  p*math.log(1+b*0.5*f)+(1-p)*math.log(1-0.5*f)
    print("全凯利预期收益：",maxf)
    print("半凯利预期收益：",halfmaxf)





# -*- coding: utf-8 -*-
# @Time    : 2019/1/23 11:26
# @Author  : hc
# @Site    :
# @File    : qts.py
# @Software: PyCharm
#!/usr/bin/env python
"""
Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
import sys
import time
import struct
import datetime
import threading

from twisted.internet import defer, reactor
from stompest.config import StompConfig
from stompest.async import Stomp
from stompest.async.listener import SubscriptionListener
from common import log


def log_w(str_text):
    stime = datetime.datetime.now().strftime("%Y%m%d %H:%M:%S.%f")
    day = datetime.datetime.now().strftime("%Y%m%d")
    str_text = stime + "\t" + str_text
    print(str_text)
    log.WriteLog(day + "_qts_Trace", str_text)


def log_e(str_text):
    stime = datetime.datetime.now().strftime("%Y%m%d %H:%M:%S.%f")
    day = datetime.datetime.now().strftime("%Y%m%d")
    str_text = stime + "\t" + str_text
    print(str_text)
    log.WriteLog(day + "_qts_err", str_text)


user = os.getenv('ACTIVEMQ_USER') or 'admin'
password = os.getenv('ACTIVEMQ_PASSWORD') or 'password'
host_backup = os.getenv('ACTIVEMQ_HOST') or '39.108.169.211'
host1 = os.getenv('ACTIVEMQ_HOST') or '39.108.169.211'
port = int(os.getenv('ACTIVEMQ_PORT') or 61613)
destination = {
    "quotation": r'/topic/Quotation1',  # 行情地址
    "tran_sh": r'/topic/sh1',            # 上海逐笔成交
    "tran_sz": r'/topic/sz1',            # 深圳逐笔成交
    "orders": r'/topic/orders1',         # 逐笔委托，只有深圳
}


PACK_TYPE_BYTE = 2
PACK_TYPE_SHORT = 3
PACK_TYPE_INT = 4
PACK_TYPE_LONG = 5
PACK_TYPE_FLOAT = 6
PACK_TYPE_DOUBLE = 7
PACK_TYPE_STRING = 8
PACK_TYPE_BYTES = 9


class stream_reader(object):
    def __init__(self, data):
        self.index = 0
        self.data = data

    def isEnd(self):
        return self.index >= len(self.data)

    def read(self, type, length=0):
        value = None
        if type == PACK_TYPE_BYTE:
            value, = struct.unpack("<B", self.data[self.index:self.index + 1])
            self.index += 1
        elif type == PACK_TYPE_SHORT:
            value, = struct.unpack("<H", self.data[self.index:self.index + 2])
            self.index += 2
        elif type == PACK_TYPE_LONG:
            value, = struct.unpack("<q", self.data[self.index:self.index + 8])
            self.index += 8
        elif type == PACK_TYPE_INT:
            value, = struct.unpack("<I", self.data[self.index:self.index + 4])
            self.index += 4
        elif type == PACK_TYPE_FLOAT:
            value, = struct.unpack("<f", self.data[self.index:self.index + 4])
            self.index += 4
        elif type == PACK_TYPE_DOUBLE:
            value, = struct.unpack("<d", self.data[self.index:self.index + 8])
            self.index += 8
        elif type == PACK_TYPE_STRING:
            value, = struct.unpack("<%ds" %
                                   length, self.data[self.index:self.index + length])
            self.index += length
        elif type == PACK_TYPE_BYTES:
            length, = struct.unpack("<H", self.data[self.index:self.index + 2])
            self.index += 2
            value, = struct.unpack("<%ds" %
                                   length, self.data[self.index:self.index + length])
        return value

def read_quo10(s):
    data = {}
    data["code"] = '%06d' % s.read(PACK_TYPE_INT)
    data["date"] = s.read(PACK_TYPE_INT)
    data["JTlocaltime"] = s.read(PACK_TYPE_INT)
    data["local_time"] = s.read(PACK_TYPE_INT)
    data["time"] = s.read(PACK_TYPE_INT)
    data["last_price"] = round(s.read(PACK_TYPE_FLOAT), 2)
    data["open_price"] = round(s.read(PACK_TYPE_FLOAT), 2)
    data["high_price"] = round(s.read(PACK_TYPE_FLOAT), 2)
    data["low_price"] = round(s.read(PACK_TYPE_FLOAT), 2)
    data["pre_close_price"] = round(s.read(PACK_TYPE_FLOAT), 2)
    for i in range(1, 11):
        data["buy_price%02d" % i] = round(s.read(PACK_TYPE_FLOAT), 2)
    for i in range(1, 11):
        data["buy_volume%02d" % i] = s.read(PACK_TYPE_INT)
    for i in range(1, 11):
        data["sell_price%02d" % i] = round(s.read(PACK_TYPE_FLOAT), 2)
    for i in range(1, 11):
        data["sell_volume%02d" % i] = s.read(PACK_TYPE_INT)
    data["up_limit"] = s.read(PACK_TYPE_FLOAT)
    data["down_limit"] = s.read(PACK_TYPE_FLOAT)
    flag = s.read(PACK_TYPE_STRING, 8)  # SecurityPhaseTag
    status = s.read(PACK_TYPE_STRING, 8)  # TradeStatus只适用上交所
    data['flag'] = 0  # todo 按需要解析
    data['status'] = 0
    return data


class Listener(object):
    def __init__(self, strategy=None, active_subscribe=None):
        self.strategy = strategy
        self.client = None
        self.count = 0                              # 接受数据条数计数 用于显示
        self.active_subscribe = active_subscribe    # 需要订阅的接口
        self.subscribe_conf = {                     # 回调函数
            "quotation": self.handleFrame,          # 行情地址
            "tran_sh": self.handleFrame_tran_sh,    # 上海逐笔成交
            "tran_sz": self.handleFrame_tran_sz,            # 深圳逐笔成交
            "orders": self.handleFrame_order,         # 逐笔委托，只有深圳
        }

    def load_subscribe(self):
        for idx, _topic in enumerate(self.active_subscribe):
            print(idx, _topic)
            self.client.subscribe(
                destination[_topic],
                listener=SubscriptionListener(
                    self.subscribe_conf[_topic]),
                headers={
                    'ack': 'auto',
                    'id': "han"+str(idx),


                })
            log_text = "  订阅  %s " % destination[_topic]
            print(log_text)
            log_w(log_text)
        # self.client.subscribe('/topic/Quotation', listener=SubscriptionListener(self.handleFrame),
        #                  headers={'ack': 'auto', 'id': 'required-for-STOMP-1.1'})

    @defer.inlineCallbacks
    def handleFrame1(self, client, frame):
        # print(self.count)
        self.count += 1
        if self.count % 10 == 0:
            print(self.count, frame.body)
        if self.count == 100000:
            self.stop(client)
    @defer.inlineCallbacks
    def run(self):
        try:
            self.strategy.startup()  # 策略当天初始化
            """
            config = StompConfig(
                'failover:(tcp://%s:%d,tcp://%s:%d)?randomize=false,startupMaxReconnectAttempts=-1,startupMaxReconnectAttempts=-1,priorityBackup=true' %
                (host1, port,host_backup, port), login=user, passcode=password, version='1.1')
            self.client = Stomp(config)
            # yield self.client.connect(headers = {"client-id":"hancheng"})
            yield self.client.connect(host='mybroker')
            """
            config = StompConfig(
                'failover:(tcp://%s:%d,tcp://%s:%d)?randomize=false,startupMaxReconnectAttempts=-1,startupMaxReconnectAttempts=-1,priorityBackup=true' %
                (host1, port, host1, port), login=user, passcode=password, version='1.1')
            self.client = Stomp(config)
            a = yield self.client.connect(host='mybroker',headers = {"client-id":"hancheng_mac_qts_quo"})
            print(a)
            self.count = 0
            start = time.time()
            print(start)

            self.load_subscribe()
        except Exception as e:
            log_e("[FUNC_name]%s\t" % sys._getframe().f_code.co_name + str(e))


    
    @defer.inlineCallbacks
    def handleFrame(self, client, frame):
        self.count += 1
        s = stream_reader(frame.body)
        while not s.isEnd():
            # data = {}
            # data["symbol"] = s.read(PACK_TYPE_INT)  # 证券代码
            # data["date"] = s.read(PACK_TYPE_INT)  # date:交易日期
            # data["JTlocaltime"] = s.read(PACK_TYPE_INT)  # 津泰采集时间
            # data["localTime"] = s.read(PACK_TYPE_INT)  # locatTime:接收时间
            # data["time"] = s.read(PACK_TYPE_INT)   # time:真实数据生成时间
            # data["last_price"] = s.read(PACK_TYPE_DOUBLE)
            # data["open_price"] = s.read(PACK_TYPE_DOUBLE)
            # data["high_price"] = s.read(PACK_TYPE_DOUBLE)
            # data["low_price"] = s.read(PACK_TYPE_DOUBLE)
            # data["pre_close_price"] = s.read(PACK_TYPE_DOUBLE)
            # data["buy_price1"] = s.read(PACK_TYPE_DOUBLE)
            # data["buy_volume1"] = s.read(PACK_TYPE_LONG)
            # data["sell_price1"] = s.read(PACK_TYPE_DOUBLE)
            # data["sell_volume1"] = s.read(PACK_TYPE_LONG)
            # data["uplimit"] = s.read(PACK_TYPE_DOUBLE)
            # data["downlimit"] = s.read(PACK_TYPE_DOUBLE)
            # data["flag"] = s.read(PACK_TYPE_STRING, 8)  # SecurityPhaseTag
            # data["status"] = s.read(PACK_TYPE_STRING, 8)  # TradeStatus只适用上交所

            data = read_quo10(s)
            if self.strategy:
                self.strategy.on_tick(data)

    @defer.inlineCallbacks
    def handleFrame_order(self, client, frame):
        self.count += 1
        s = stream_reader(frame.body)
        while not s.isEnd():
            data = {}
            data["set_id"] = s.read(PACK_TYPE_INT)
            data["rec_id"] = s.read(PACK_TYPE_LONG)
            data["code"] = s.read(PACK_TYPE_INT)
            data["order_time"] = s.read(PACK_TYPE_LONG)
            data["order_price"] = s.read(PACK_TYPE_DOUBLE)
            data["order_volume"] = s.read(PACK_TYPE_DOUBLE)
            data["order_code"] = s.read(PACK_TYPE_BYTE)
            data["reverse1"] = s.read(PACK_TYPE_BYTE)
            data["order_type"] = s.read(PACK_TYPE_BYTE)
            data["reverse2"] = s.read(PACK_TYPE_BYTE)
            if self.strategy:
                self.strategy.on_order(data)

    @defer.inlineCallbacks
    def handleFrame_tran_sz(self, client, frame):
        s = stream_reader(frame.body)
        while not s.isEnd():
            data = {}
            data["set_id"] = s.read(PACK_TYPE_INT)         #     频道代码
            data["rec_id"] = s.read(PACK_TYPE_LONG)        #     消息记录号 从 1 开始计数，同一频道连续。
            data["buy_order_id"] = s.read(PACK_TYPE_LONG) #     买方委托索引：从 1 开始计数,0 表示无对应委托。
            data["sel_order_id"] = s.read(PACK_TYPE_LONG) #     卖方委托索引：从 1 开始计数,0 表示无对应委托。
            data["code"] = s.read(PACK_TYPE_INT)
            data["trade_time"] = s.read(PACK_TYPE_LONG)
            data["trade_price"] = s.read(PACK_TYPE_DOUBLE)
            data["trade_volume"] = s.read(PACK_TYPE_DOUBLE)
            data["trade_type"] = s.read(PACK_TYPE_BYTE)   #     ASCII码 成交类别：4=撤销，主动或自动撤单执行报告；F=成交，成交执行报告。
            data["res"] = s.read(PACK_TYPE_BYTE)
            if self.strategy:
                self.strategy.on_tran_sz(data)

    @defer.inlineCallbacks
    def handleFrame_tran_sh(self, client, frame):
        s = stream_reader(frame.body)
        while not s.isEnd():
            data = {}
            data["trade_channel"] = s.read(PACK_TYPE_INT)  # 成交通道
            data["rec_id"] = s.read(PACK_TYPE_INT)         #    业务索引
            data["buy_order_id"] = s.read(PACK_TYPE_LONG)    #    买方订单号
            data["sel_order_id"] = s.read(PACK_TYPE_LONG)      #  卖方订单号
            data["code"] = s.read(PACK_TYPE_INT)
            data["trade_time"] = s.read(PACK_TYPE_INT)
            data["trade_price"] = s.read(PACK_TYPE_DOUBLE)
            data["trade_volume"] = s.read(PACK_TYPE_INT)
            data["buysell_flag"] = s.read(PACK_TYPE_BYTE) #  ASCII码 B–外盘,主动买；S –内盘,主动卖；N – 未知。
            data["res"] = s.read(PACK_TYPE_BYTE)
            if self.strategy:
                self.strategy.on_tran_sh(data)

    @defer.inlineCallbacks
    def stop(self):
        print('Disconnecting. Waiting for RECEIPT frame ...',)
        yield self.client.disconnect(receipt='bye')
        print('ok')
        print('Received %s frames' % (self.count))

# 计算时间的秒数差距
def calc_diff_time(beg,end=None):
    b = datetime.datetime.strptime(str(beg), "%Y%m%d%H%M%S%f")
    if end:
        e = datetime.datetime.strptime(str(end), "%Y%m%d%H%M%S%f")
    else:
        e = datetime.datetime.now()
    seconds_diff = (e - b).total_seconds()
    return seconds_diff

class tst_t:
    def __init__(self):
        self.name = "测试策略"
        self.cnt = 0
        self.total = 10  # 达到累计数时打印一条

    def startup(self):
        pass
    def on_tick(self, tick):

        self.cnt += 1
        # print(self.cnt,tick)
        if (self.cnt % self.total == 0):
            _time = tick["time"]
            _date = int(datetime.datetime.now().strftime('%Y%m%d'))
            _datetime = str(_date) + str(_time)
            diff = calc_diff_time(_datetime)
            print("行情时间[%s]qts接收时间[%s]本机时间[%s]股票id[%s]与本机时间间隔[%s]"
                  % (_datetime,
                     tick["local_time"],
                     datetime.datetime.now(),
                     tick["code"]
                     , diff))

    def on_order(self, tick):
        self.cnt += 1
        if (self.cnt % self.total == 0):
            print("on_order", tick)

    def on_tran_sz(self, tick):
        self.cnt += 1
        if (self.cnt % self.total == 0):
            print("on_tran_sz", tick)

    def on_tran_sh(self, tick):
        self.cnt += 1
        if (self.cnt % self.total == 0):
            print("on_tran_sh", tick)


if __name__ == '__main__':
    DAY_START = datetime.time(9, 10)  # 日盘启动和停止时间
    DAY_END = datetime.time(23, 30)
    # DAY_START1 = datetime.time(10,7)  # 日盘启动和停止时间
    # DAY_END1 = datetime.time(15, 30)
    # quotation
    # tran_sh
    # tran_sz
    # orders
    # import Strategy_Limit_UP
    sub_list = ("quotation","tran_sh","tran_sz","orders")
    # listener = Listener(Strategy_Limit_UP.Strategy_Limit_UP(), sub_list)
    try:
        # sub_list = ("quotation",)

        #from gateway import listener
        #ltn = listener.Listener()
        ltn = Listener(tst_t(), sub_list)
        ltn.run()
    except Exception as e:
        print(e)
    pass
    """
    def action(listener):
        p = False  # 定时器线程。用来控制关闭
        while True:
            print("111")
            recording = False
            # weekday = datetime.datetime.now().weekday()
            currentTime = datetime.datetime.now().time()
            # 判断当前处于的时间段
            if currentTime >= DAY_START and currentTime <= DAY_END:
                recording = True

            print("是否为交易时间[%s]" % recording)
            # 记录时间则需要启动子进程
            print('[%s] 定时器线程 name is:%s\r' %
                  (currentTime, threading.currentThread().getName()))
            # print(listener.client.session)
            if recording and not p:
                ltn.run()
                p = True
            if not recording and p:
                ltn.stop()
                p = False
            time.sleep(20)
    p = threading.Thread(target=action, args=(ltn,))
    p.start()
    """
    reactor.run()






#!/usr/bin/env python
"""
Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
import sys
import time

from twisted.internet import defer, reactor

from stompest.config import StompConfig
from stompest.async import Stomp
from stompest.async.listener import SubscriptionListener

user = os.getenv('ACTIVEMQ_USER') or 'admin'
password = os.getenv('ACTIVEMQ_PASSWORD') or 'password'
host_backup = os.getenv('ACTIVEMQ_HOST') or '39.108.169.211'
host1 = os.getenv('ACTIVEMQ_HOST') or '39.108.169.211'
# host_backup = os.getenv('ACTIVEMQ_HOST') or '192.168.1.188'
# host1 = os.getenv('ACTIVEMQ_HOST') or '192.168.1.188'
port = int(os.getenv('ACTIVEMQ_PORT') or 61613)
destination = sys.argv[1:2] or ['/topic/event']
destination = destination[0]

messages = 10000


class Listener(object):
    @defer.inlineCallbacks
    def run(self):
        # config = StompConfig('tcp://%s:%d' % (host1, port), login=user, passcode=password, version='1.1')

        config = StompConfig(
            'failover:(tcp://%s:%d,tcp://%s:%d)?randomize=false,startupMaxReconnectAttempts=-1,startupMaxReconnectAttempts=-1,priorityBackup=true' %
            (host1, port, host_backup, port), login=user, passcode=password, version='1.1')
        self.client = Stomp(config)
        a = yield self.client.connect(host='mybroker',headers = {"client-id":"hancheng_mac_lst"})
        print(a)
        self.count = 0
        self.start = time.time()
        print(self.start)
        self.client.subscribe(destination, listener=SubscriptionListener(self.handleFrame), headers={'ack': 'auto', 'id': 'required-for-STOMP-1.1'})
        
    @defer.inlineCallbacks
    def handleFrame(self, client, frame):
        # print(self.count)
        self.count += 1
        if self.count%10==0:
            print(self.count,frame.body)
        if self.count == messages:
            self.stop(client)
    
    @defer.inlineCallbacks
    def stop(self):
        print( 'Disconnecting. Waiting for RECEIPT frame ...',)
        yield self.client.disconnect(receipt='bye')
        print ('ok')
        
        diff = time.time() - self.start
        print ('Received %s frames in %f seconds' % (self.count, diff))
        reactor.stop()


if __name__ == '__main__':

    def aSillyBlockingMethod(x,l):
        n=0
        while 1:
            import time
            time.sleep(3)
            print(x)
            print("n",n)
            n+=1
            if n==2:
                l.stop()
            if n == 10:
                print("111")
                l.run()
                print("2222")
    from threading import Thread
    l = Listener()
    # a = Thread(target=aSillyBlockingMethod,args=["2 secodns have passed",l ])
    # a.start()
    reactor.callInThread(aSillyBlockingMethod,"2 secodns have passed", l)
    l.run()
    reactor.run()